@@ -37,9 +37,9 @@ extern time_t get_date(const char *, const time_t *);
 static time_t ydhms_tm_diff(int, int, int, int, int, const struct tm *);
 static time_t my_mktime(struct tm *);
 
-long convtoyearsecs(char *date)
+time_t convtoyearsecs(char *date)
 {
-    long yearsecs;
+    time_t yearsecs;
     char *p, *s = date;
 
     /* the (non-standard) timezone specs GMT0 and BST-1
@@ -76,13 +76,15 @@ long convtoyearsecs(char *date)
 
 char *getlocaltime(void)
 {
-    static char s[DATESTRLEN];
+    static char s[DATESTRLEN + 5];
     time_t tp;
     struct tm *tmptr;
 
     time(&tp);
     tmptr = (set_gmtime ? gmtime(&tp) : localtime(&tp));
 
+    s[0] = '\0';
+
     if (set_dateformat != NULL) {
 	strftime(s, DATESTRLEN, set_dateformat, tmptr);
         /* 
@@ -200,7 +202,7 @@ char *secs_to_iso_meta(time_t t)
      * YYYY-MM-DD
      * This buffer will be overwritten by next call to secs_to_iso_meta.
      */
-    static char s[10];
+    static char s[11];
     struct tm *tm;
 
     tm = localtime(&t);
