@@ -11,18 +11,20 @@
  */
 
 #include "dmatch.h"
+#ifdef __LCC__
+#include "../lcc/config.h"
+#else
 #include "../config.h"
+#endif
 
 #ifdef HAVE_CTYPE_H
 #include <ctype.h>
 #endif
 
-#ifndef FALSE
+#undef FALSE
 #define FALSE 0
-#endif
-#ifndef TRUE
+#undef TRUE
 #define TRUE 1
-#endif
 
 char Match(char *string, char *pattern)
 {
@@ -30,7 +32,7 @@ char Match(char *string, char *pattern)
 	if (!*string)
 	    return (!*pattern);
 	if (toupper(*string) ^ toupper(*pattern) && '?'^*pattern)
-	     return FALSE;
+	     return (FALSE);
     }
 
     /* two-line patch to prevent *too* much recursiveness: */
@@ -40,8 +42,8 @@ char Match(char *string, char *pattern)
 
     do {
 	if (Match(string, pattern +1))
-	     return TRUE;
+	     return (TRUE);
     } while (*string++);
 
-    return FALSE;
+    return (FALSE);
 }
