@@ -26,7 +26,6 @@
 #include <string.h>
 #include <ctype.h>
 
-
 static int str_similar_str(const char *str1, const char *str2);
 static char *stripwhitespace(const char *line);
 static int new_reply_to = -1;
@@ -37,14 +36,13 @@ static char *get_path(struct emailinfo *ep, struct emailinfo *ep2)
 {
     char *path = "";
     if (ep2->subdir && ep2->subdir != ep->subdir)
-	trio_asprintf(&path, "%s%s", ep2->subdir->rel_path_to_top,
-		      ep2->subdir->subdir);
+       trio_asprintf(&path, "%s%s", ep2->subdir->rel_path_to_top, ep2->subdir->subdir);
     return path;
 }
 
-static struct body *place_anchor(const String_Match * match_info,
-				 struct body *bp, char *buffer, FILE *fp2,
-				 char **ptr, const char *anchor)
+static struct body *place_anchor(const String_Match * match_info, 
+                                 struct body *bp, char *buffer, FILE *fp2, 
+                                 char **ptr, const char *anchor)
 {
     if (match_info) {
 	int index;
@@ -92,9 +90,7 @@ static struct body *place_anchor(const String_Match * match_info,
     return bp;
 }
 
-static int
-place_a_end(const String_Match * match_info, struct body **bp,
-	    char *buffer, FILE *fp2, char **ptr)
+static int place_a_end(const String_Match * match_info, struct body **bp, char *buffer, FILE *fp2, char **ptr)
 {
     int index;
     char token[MAXLINE];
@@ -132,10 +128,7 @@ place_a_end(const String_Match * match_info, struct body **bp,
 ** Alter the message being quoted to add an anchor specifying the quoted text
 */
 
-static int
-add_anchor(int msgnum, int quoting_msgnum, int quote_num,
-	   const char *anchor, char *line, int find_substr,
-	   int count_quoted_lines, const String_Match * match_info)
+static int add_anchor(int msgnum, int quoting_msgnum, int quote_num, const char *anchor, char *line, int find_substr, int count_quoted_lines, const String_Match * match_info)
 {
     char *filename;
     char *tmpfilename;
@@ -170,15 +163,13 @@ add_anchor(int msgnum, int quoting_msgnum, int quote_num,
 	if (msgnum > quoting_msgnum)
 	    return 0;		/* just a forward ref */
 	if (set_showprogress)
-	    fprintf(stderr, "Couldn't read message number %d (linked from %d). "
-		    "May mean message deleted with delete_level = 0.\n",
-		    msgnum, quoting_msgnum);
+			fprintf(stderr, "Couldn't read message number %d (linked from %d). " "May mean message deleted with delete_level = 0.\n", msgnum, quoting_msgnum);
 	return -1;
     }
-    tmpfilename = htmlfilename("tmp", ep, "tmp");
+    tmpfilename = htmlfilename("tmp", ep, "tmp");	/* AUDIT biege: where is the tmp-file created? cwd? what about checking the return-value */
     if ((fp2 = fopen(tmpfilename, "w")) == NULL) {
-	sprintf(errmsg, "Couldn't write \"%s\".", tmpfilename);
-	progerr(NULL);
+	snprintf(errmsg, sizeof(errmsg), "Couldn't write \"%s\".", tmpfilename);
+	progerr(errmsg);
     }
     while (fgets(buffer, sizeof(buffer), fp1)) {
 	if ((find_substr && str_similar_str(buffer, line))
@@ -190,9 +181,7 @@ add_anchor(int msgnum, int quoting_msgnum, int quote_num,
 	        struct emailinfo *ep2;
 		if (hashnumlookup(quoting_msgnum, &ep2)) {
 		    char *path = get_path(ep, ep2);
-		    fprintf(fp2, "<a href=\"%s%.4d.%s#qlink%d\">%s</a>",
-			    path, quoting_msgnum, set_htmlsuffix,
-			    quote_num, set_link_to_replies);
+					fprintf(fp2, "<a href=\"%s%.4d.%s#qlink%d\">%s</a>", path, quoting_msgnum, set_htmlsuffix, quote_num, set_link_to_replies);
 		    if (*path)
 		        free(path);
 		}
@@ -203,8 +192,7 @@ add_anchor(int msgnum, int quoting_msgnum, int quote_num,
 		    count_quoted_lines = -1;
 		    break;	/* reply quoted more lines than exist? */
 		}
-		if (match_info
-		    && place_a_end(match_info, &bp, buffer, fp2, &ptr)) {
+		if (match_info && place_a_end(match_info, &bp, buffer, fp2, &ptr)) {
 		    wrote_a_end = TRUE;
 		    count_quoted_lines = 0;
 		    if (bp)
@@ -224,14 +212,12 @@ add_anchor(int msgnum, int quoting_msgnum, int quote_num,
 		    else
 			++count_quoted_lines;
 		}
-	    } while (--count_quoted_lines > 0
-		     && fgets(buffer, sizeof(buffer), fp1));
+	    } while (--count_quoted_lines > 0 && fgets(buffer, sizeof(buffer), fp1));
 	    if (!wrote_a_end)
 		fputs("</a>", fp2);
 	    if (count_quoted_lines > 0)	/* got eof? */
 		break;		/* avoid last fputs */
-	    else if (count_quoted_lines == 0
-		     && !fgets(buffer, sizeof(buffer), fp1))
+            else if (count_quoted_lines == 0 && !fgets(buffer, sizeof(buffer), fp1))
 		break;
 	}
 	else if (in_body && bp && (tmpptr = remove_hypermail_tags(buffer))) {
@@ -256,14 +242,12 @@ add_anchor(int msgnum, int quoting_msgnum, int quote_num,
 	remove(tmpfilename);
     else {
 	if (rename(tmpfilename, filename) == -1) {
-	    sprintf(errmsg, "Couldn't rename \"%s\" to %s.",
-		    tmpfilename, filename);
-	    progerr(NULL);
+	    snprintf(errmsg, sizeof(errmsg), "Couldn't rename \"%s\" to %s.", tmpfilename, filename);
+	    progerr(errmsg);
 	}
 	if (chmod(filename, set_filemode) == -1) {
-	    sprintf(errmsg, "Couldn't chmod \"%s\" to %o.",
-		    filename, set_filemode);
-	    progerr(NULL);
+	    snprintf(errmsg, sizeof(errmsg), "Couldn't chmod \"%s\" to %o.", filename, set_filemode);
+	    progerr(errmsg);
 	}
     }
     free(filename);
@@ -291,36 +275,27 @@ static char *unquote_and_strip(char *line)
 /*
 ** Find URL of the message this line of quoted text was taken from
 */
-/* #define MAX_ANCHOR_LEN 128 */
 
-static char *url_replying_to(struct emailinfo *email,
-			     char *line1,	/* first line of quoted text, with html */
+static char *url_replying_to(struct emailinfo *email, char *line1,	/* first line of quoted text, with html */
 			     const char *line2,	/* first line of quoted text, w/o html */
-			     const struct body *bp,
-			     int quote_num,
-			     int *quoting_msgnum,
-			     int count_quoted_lines,
-			     int maybe_reply)
+			     const struct body *bp, int quote_num, int *quoting_msgnum, int count_quoted_lines, int maybe_reply)
 {
     String_Match match_info;
     char *p;
     int subjmatch = 0;
     char *anchor;
     struct emailinfo *ep;
-    int statusnum =
-	hashreplynumlookup(*quoting_msgnum, email->inreplyto, email->subject,
+	int statusnum = hashreplynumlookup(*quoting_msgnum, email->inreplyto, email->subject,
 			   &subjmatch);
     hashnumlookup(*quoting_msgnum, &ep);
     trio_asprintf(&anchor, "%.4dqlink%d", *quoting_msgnum, quote_num);
     if (statusnum != -1) {
 	struct emailinfo *ep2;
 	hashnumlookup(statusnum, &ep2);
-	if (add_anchor(statusnum, *quoting_msgnum, quote_num,
-		       anchor, line1, 0, count_quoted_lines, NULL)) {
+		if (add_anchor(statusnum, *quoting_msgnum, quote_num, anchor, line1, 0, count_quoted_lines, NULL)) {
 	    char *path = get_path(ep, ep2);
 	    char *buf;
-	    trio_asprintf(&buf, "%s%.4d.%s#%s",
-			  path, statusnum, set_htmlsuffix, anchor);
+			trio_asprintf(&buf, "%s%.4d.%s#%s", path, statusnum, set_htmlsuffix, anchor);
 	    if (maybe_reply)
 		set_new_reply_to(statusnum, strlen(line2));
 	    if (*path)
@@ -330,20 +305,18 @@ static char *url_replying_to(struct emailinfo *email,
 	}
 	if (strlen(line2) > 6 && (p = strstr(line2, "..."))) {
 	    char *parsed;
-	    char *tptr = (char *)emalloc(p - line2 + 1 + strlen(p + 3));
+	    char *tptr = (char *)emalloc(p - line2 + 1 + strlen(p + 3));	/* AUDIT biege: IOF unlikely */
 	    strncpy(tptr, line2, p - line2);
 	    strcpy(tptr + (p - line2), p + 3);
-	    parsed = ConvURLsString(tptr, email->msgid, email->subject);
+	    parsed = ConvURLsString(tptr, email->msgid, email->subject, email->charset);
 	    free(tptr);
 	    tptr = stripwhitespace(parsed ? parsed : "");
 	    if (parsed)
 		free(parsed);
-	    if (add_anchor(statusnum, *quoting_msgnum, quote_num, 
-			   anchor, tptr, 1, count_quoted_lines, NULL)) {
+			if (add_anchor(statusnum, *quoting_msgnum, quote_num, anchor, tptr, 1, count_quoted_lines, NULL)) {
 	        char *path = get_path(ep, ep2);
 		char *buf;
-		trio_asprintf(&buf, "%s%.4d.%s#%s",
-			      path, statusnum, set_htmlsuffix, anchor);
+				trio_asprintf(&buf, "%s%.4d.%s#%s", path, statusnum, set_htmlsuffix, anchor);
 		free(tptr);
 		if (maybe_reply)
 		    set_new_reply_to(statusnum, strlen(buf));
@@ -372,28 +345,25 @@ static char *url_replying_to(struct emailinfo *email,
 	    free(stripped);
 	    PushString(&exact_line, unquote(bp->line));
 	}
-	search_for_quote(PUSH_STRING(full_line), PUSH_STRING(exact_line),
-			 *quoting_msgnum, &match_info);
+		search_for_quote(PUSH_STRING(full_line), PUSH_STRING(exact_line), *quoting_msgnum, &match_info);
 	free(PUSH_STRING(full_line));
 	free(PUSH_STRING(exact_line));
     }
     if (match_info.msgnum >= 0) {
-	char *parsed =
-	    ConvURLsString(match_info.last_matched_string, email->msgid,
-			   email->subject);
+		char *parsed = ConvURLsString(match_info.last_matched_string, email->msgid,
+			   email->subject, email->charset);
 	if (parsed) {
 	    char *parsed2 = stripwhitespace(parsed);
 	    free(parsed);
-	    if (add_anchor(match_info.msgnum, *quoting_msgnum, quote_num,
-		 anchor, parsed2, 1, count_quoted_lines, &match_info)) {
+	    if (add_anchor(match_info.msgnum, *quoting_msgnum, quote_num, anchor, parsed2, 1, count_quoted_lines, &match_info)) {
 	        struct emailinfo *ep2;
-		struct body *bp0 = hashnumlookup(match_info.msgnum, &ep2);
-		char *path = get_path(ep, ep2);
-		char *buf;
-		trio_asprintf(&buf, "%s%.4d.%s#%s", path,
-			      match_info.msgnum, set_htmlsuffix, anchor);
-		set_new_reply_to(match_info.msgnum,
-				 match_info.match_len_bytes);
+                char *path;
+                char *buf;
+                hashnumlookup(match_info.msgnum, &ep2);
+                path = get_path(ep, ep2);
+
+		trio_asprintf(&buf, "%s%.4d.%s#%s", path, match_info.msgnum, set_htmlsuffix, anchor);
+		set_new_reply_to(match_info.msgnum, match_info.match_len_bytes);
 		free(parsed2);
 		if (*path)
 		    free(path);
@@ -417,18 +387,16 @@ static char *url_replying_to(struct emailinfo *email,
 ** of another message
 */
 
-int
-handle_quoted_text(FILE *fp, struct emailinfo *email, const struct body *bp,
-		   char *line, int inquote, int quote_num,
-		   bool replace_quoted, int maybe_reply)
+int handle_quoted_text(FILE *fp, struct emailinfo *email, const struct body *bp, char *line, int inquote, int quote_num, bool replace_quoted, int maybe_reply)
 {
     char *url1;
     int quoting_msgnum = email->msgnum;
     const struct body *last_quoted_line = bp;
     int count_quoted_lines = 0;
-    const char *fmt2 = set_iquotes ? "<em>%s</em><br>" : "%s<br>";
-    char *cvtd_line = ConvURLsString(unquote(line), email->msgid, email->subject);
+    char *fmt2;
+    char *cvtd_line = ConvURLsString(unquote(line), email->msgid, email->subject, email->charset);
     char *buffer1;
+	trio_asprintf(&fmt2, set_iquotes ? "<em class=\"%s\">%%s</em><br>" : "<span class=\"%s\">%%s</span><br>", find_quote_class(line));
     trio_asprintf(&buffer1, fmt2, cvtd_line ? cvtd_line : "");
     if (cvtd_line)
 	free(cvtd_line);
@@ -439,21 +407,18 @@ handle_quoted_text(FILE *fp, struct emailinfo *email, const struct body *bp,
     }
     cvtd_line = unquote_and_strip(line);
     if (strlen(cvtd_line) < 5 && (!replace_quoted || !inquote)) {
-	char *parsed = ConvURLsString(line, email->msgid, email->subject);
+	char *parsed = ConvURLsString(line, email->msgid, email->subject, email->charset);
 	if (parsed) {
 	    fprintf(fp, fmt2, parsed);
 	    free(parsed);
 	}
     }
     else if ((!inquote || !found_quote)
-	     && (url1 = url_replying_to(email, buffer1, cvtd_line,
-					bp, ++quote_num, &quoting_msgnum,
-					count_quoted_lines, maybe_reply))) {
+		 && (url1 = url_replying_to(email, buffer1, cvtd_line, bp, ++quote_num, &quoting_msgnum, count_quoted_lines, maybe_reply))) {
 	static const char *fmt1 = "<a href=\"%s\">%s</a>%s<br>\n";
 	char *tmpline;
 	char *p2;
-	bool replacing = replace_quoted && set_quote_link_string &&
-	    set_quote_link_string[0];
+		bool replacing = replace_quoted && set_quote_link_string && set_quote_link_string[0];
 	char *part2 = strcasestr(line, "<a href=");
 	if (!part2) {
 	    part2 = "";
@@ -466,11 +431,11 @@ handle_quoted_text(FILE *fp, struct emailinfo *email, const struct body *bp,
 	}
 	if (set_link_to_replies)
 	    fprintf(fp, "<a name=\"qlink%d\"></a>", quote_num);
-	p2 = ConvURLsString(part2, email->msgid, email->subject);
+	p2 = ConvURLsString(part2, email->msgid, email->subject, email->charset);
 	if (replacing)
 	    fprintf(fp, fmt1, url1, set_quote_link_string, p2 ? p2 : "");
 	else {
-	    char *tmpptr = convchars(tmpline);
+	    char *tmpptr = convchars(tmpline, email->charset);
 	    if (tmpptr) {
 		fprintf(fp, fmt1, url1, tmpptr, p2 ? p2 : "");
 		free(tmpptr);
@@ -482,10 +447,11 @@ handle_quoted_text(FILE *fp, struct emailinfo *email, const struct body *bp,
 	    free(p2);
 	free(cvtd_line);
 	free(buffer1);
+	free(fmt2);
 	return 1;
     }
     else if (!replace_quoted || !inquote) {
-	char *parsed = ConvURLsString(bp->line, email->msgid, email->subject);
+	char *parsed = ConvURLsString(bp->line, email->msgid, email->subject, email->charset);
 	if (parsed) {
 	    fprintf(fp, quoting_msgnum >= 0 ? fmt2 : "%s<br>\n", parsed);
 	    free(parsed);
@@ -493,6 +459,7 @@ handle_quoted_text(FILE *fp, struct emailinfo *email, const struct body *bp,
     }
     free(cvtd_line);
     free(buffer1);
+    free(fmt2);
     return 0;
 }
 
@@ -520,8 +487,7 @@ void set_new_reply_to(int msgnum, int match_len)
     }
     else {
 	if (msgnum == alt_msgnum || alt_msgnum == -1) {
-	    if (++count_alt > count_last
-		|| (count_alt == count_last && match_len > best_match_len)) {
+			if (++count_alt > count_last || (count_alt == count_last && match_len > best_match_len)) {
 		count_last = count_alt;
 		new_reply_to = msgnum;
 		if (match_len > best_match_len)
@@ -544,9 +510,7 @@ int get_new_reply_to()
  * "In reply to"
 */
 
-void
-replace_maybe_replies(const char *filename, struct emailinfo *ep,
-		      int new_reply_to)
+void replace_maybe_replies(const char *filename, struct emailinfo *ep, int new_reply_to)
 {
     char tmpfilename[MAXFILELEN];
     char buffer[MAXLINE];
@@ -558,14 +522,14 @@ replace_maybe_replies(const char *filename, struct emailinfo *ep,
 
     if (!hashnumlookup(new_reply_to, &ep2))
 	return;
-    sprintf(tmpfilename, "%s/aaaa.tmp", set_dir);
+    snprintf(tmpfilename, sizeof(tmpfilename), "%s/aaaa.tmp", set_dir);	/* AUDIT biege: poss. BOF. */
     if ((fp1 = fopen(filename, "r")) == NULL) {
-	sprintf(errmsg, "Couldn't read \"%s\".", filename);
-	progerr(NULL);
+        snprintf(errmsg, sizeof(errmsg), "Couldn't read \"%s\".", filename);
+	progerr(errmsg);
     }
     if ((fp2 = fopen(tmpfilename, "w")) == NULL) {
-	sprintf(errmsg, "Couldn't write \"%s\".", tmpfilename);
-	progerr(NULL);
+        snprintf(errmsg, sizeof(errmsg), "Couldn't write \"%s\".", tmpfilename);
+	progerr(errmsg);
     }
     while (fgets(buffer, sizeof(buffer), fp1)) {
 	if (strstr(buffer, "<!-- body=\"end\" -->\n"))
@@ -573,19 +537,39 @@ replace_maybe_replies(const char *filename, struct emailinfo *ep,
 	if (!strcmp(buffer, "<!-- body=\"start\" -->\n"))
 	    in_body = 1;
 	if (!in_body) {
-	    if (!strcmp(buffer, "<!-- nextthread=\"start\" -->\n")) {
-		char *tmpptr = convchars(ep2->subject);
+	  if (!strcmp(buffer, "<!-- unextthread=\"start\" -->\n")) {
+		char *tmpptr = convchars(ep2->subject, ep2->charset);
 		if (tmpptr) {
 		    char *path = get_path(ep, ep2);
-		    fprintf(fp2, "<li> <strong>%s:</strong> "
-			    "<a href=\"%s%.4d.%s\">%s: \"%s\"</a>\n",
-			    lang[MSG_IN_REPLY_TO], path, new_reply_to,
-			    set_htmlsuffix, ep2->name,
-			    tmpptr ? tmpptr : "");
+                    fprintf(fp2,"[ <a href=\"%s%.4d.%s\" title=\"%s: &quot;%s&quot;\">%s</a> ]\n", 
+			    path, new_reply_to, set_htmlsuffix, lang[MSG_LTITLE_IN_REPLY_TO], 
+			    ep2->name, tmpptr ? tmpptr : "");
 		    free(tmpptr);
 		}
-	    }
-	    else {
+	  }
+	  else
+	  if (!strcmp(buffer, "<!-- lnextthread=\"start\" -->\n")) {
+	        char *tmpptr = convchars(ep2->subject, ep2->charset);
+		if (tmpptr) {
+		    char *path = get_path(ep, ep2);
+                    fprintf(fp2, "<li><dfn>%s</dfn> " 
+			    "<a href=\"%s%.4d.%s\" title=\"%s\">%s: \"%s\"</a></li>\n", 
+			    lang[MSG_IN_REPLY_TO], path, 
+			    new_reply_to, set_htmlsuffix, lang[MSG_LTITLE_IN_REPLY_TO], 
+			    ep2->name, tmpptr ? tmpptr : "");
+		    free(tmpptr);
+		}
+	  }
+	  else
+	  if (!strcmp(buffer, "<!-- nextthread=\"start\" -->\n")) {
+		char *tmpptr = convchars(ep2->subject, ep2->charset);
+		if (tmpptr) {
+		    char *path = get_path(ep, ep2);
+                    fprintf(fp2, "<li> <strong>%s:</strong> " "<a href=\"%s%.4d.%s\">%s: \"%s\"</a>\n", lang[MSG_IN_REPLY_TO], path, new_reply_to, set_htmlsuffix, ep2->name, tmpptr ? tmpptr : "");
+		    free(tmpptr);
+		}
+	  }
+	  else {
 		static const char *patts[] = {
 		    "<b>Maybe in reply to:</b>",
 		    "<strong>%s:</strong>",
@@ -593,9 +577,18 @@ replace_maybe_replies(const char *filename, struct emailinfo *ep,
 		    "<strong>%s:</strong>",
 		    "<li> <b>Previous message:</b> <a href=\"",
 		    "<li> <strong>%s:</strong> <a href=\"",
+		    "<li><dfn>%s</dfn>: <a href=\"",
+		    "<li><dfn>%s</dfn>: <a href=\"",
+		    "<li><dfn>%s</dfn>: <a href=\"",
 		    NULL
 		};
 		static const int indices[] = { MSG_MAYBE_IN_REPLY_TO,
+		    MSG_MAYBE_IN_REPLY_TO,
+		    MSG_IN_REPLY_TO,
+		    MSG_IN_REPLY_TO,
+		    MSG_PREVIOUS_MESSAGE,
+		    MSG_PREVIOUS_MESSAGE,
+		    MSG_MAYBE_IN_REPLY_TO,
 		    MSG_IN_REPLY_TO,
 		    MSG_PREVIOUS_MESSAGE
 		};
@@ -603,10 +596,9 @@ replace_maybe_replies(const char *filename, struct emailinfo *ep,
 		int surpress = 0;
 		for (i = 0; patts[i]; ++i) {
 		    char temp[256];
-		    sprintf(temp, patts[i], lang[indices[i / 2]]);
+		    snprintf(temp,sizeof(temp), patts[i], lang[indices[i]]);
 		    if ((ptr = strcasestr(buffer, temp))
-			&& (i < 4 || new_reply_to == atoi(ptr + strlen(temp))))
-		    {
+			&& (i < 4 || new_reply_to == atoi(ptr + strlen(temp)))) {
 			surpress = 1;
 			break;
 		    }
@@ -627,9 +619,8 @@ replace_maybe_replies(const char *filename, struct emailinfo *ep,
     fclose(fp2);
 
     if (rename(tmpfilename, filename) == -1) {
-	sprintf(errmsg, "Couldn't rename \"%s\" to %s.",
-		tmpfilename, filename);
-	progerr(NULL);
+	snprintf(errmsg, sizeof(errmsg), "Couldn't rename \"%s\" to %s.", tmpfilename, filename);
+	progerr(errmsg);
     }
 }
 
