@@ -22,7 +22,13 @@
 #define MAIN_FILE 1
 
 #include "hypermail.h"
+
+#ifdef __LCC__
+#include "../lcc/defaults.h"
+#else
 #include "defaults.h"
+#endif
+
 #include "setup.h"
 #include "parse.h"
 #include "print.h"
@@ -44,7 +50,6 @@ extern int opterr;
 /*
 ** Prints user startup errors stops. 
 */
-
 void cmderr(char *errorstr)
 {
     fprintf(stderr, "\n%s: %s: %s\n", lang[MSG_ERROR], PROGNAME, errorstr);
@@ -109,8 +114,7 @@ static char *setindex(char *dfltindex, char *indextype, char *suffix)
 
 void version(void)
 {
-    printf("%s: %s: %s %s: %s\n", PROGNAME,
-	   lang[MSG_VERSION], VERSION, lang[MSG_PATCHLEVEL], PATCHLEVEL);
+    printf("%s: %s: %s %s: %s\n", PROGNAME, lang[MSG_VERSION], VERSION, lang[MSG_PATCHLEVEL], PATCHLEVEL);
     exit(0);
 }
 
@@ -197,8 +201,8 @@ int main(int argc, char **argv)
 #define GETOPT_OPTSTRING ("a:Ab:c:d:gil:L:m:n:o:ps:tTuvVxX0:1M?")
 
     /* get pre config options here */
-    while ((i = getopt(argc,argv, GETOPT_OPTSTRING )) != -1) {
-        switch((char) i) {
+	while ((i = getopt(argc, argv, GETOPT_OPTSTRING)) != -1) {
+		switch ((char)i) {
 	case 'c':
 	    configfile = strreplace(configfile, optarg);
 	    break;
@@ -207,8 +211,7 @@ int main(int argc, char **argv)
 	    break;
 	case 'V':
 	    version();
-	 /*NOTREACHED*/ 
-	case 'a':
+		 /*NOTREACHED*/ case 'a':
 	case 'A':
 	case 'b':
 	case 'd':
@@ -253,8 +256,8 @@ int main(int argc, char **argv)
 
     /* now get the post-config options! */
 
-    while ((i = getopt(argc,argv, GETOPT_OPTSTRING)) != -1) {
-        switch((char) i) {
+	while ((i = getopt(argc, argv, GETOPT_OPTSTRING)) != -1) {
+		switch ((char)i) {
 	case 'A':
 	    set_append = 1;
 	    break;
@@ -324,6 +327,9 @@ int main(int argc, char **argv)
 	case 'M':
 	    set_usemeta = TRUE;
 	    break;
+	case 'N':
+ 	    set_nonsequential = TRUE;
+	    break;
 	case '?':
 	default:
 	    break;
@@ -354,16 +360,14 @@ int main(int argc, char **argv)
 	locale_code = NULL;
 
     if ((tlang = valid_language(set_language, &locale_code)) == NULL) {
-	sprintf(errmsg, "\"%s\" %s.", set_language,
-		lang[MSG_LANGUAGE_NOT_SUPPORTED]);
+	snprintf(errmsg, sizeof(errmsg), "\"%s\" %s.", set_language, lang[MSG_LANGUAGE_NOT_SUPPORTED]);
 	cmderr(errmsg);
     }
 
 #ifdef HAVE_LOCALE_H
-    if ( ! setlocale(LC_ALL, locale_code) ) {
-        sprintf(errmsg, "WARNING: locale \"%s\", not supported.\n",
-		locale_code);
-        fprintf(stderr, errmsg);
+	if (!setlocale(LC_ALL, locale_code)) {
+	    snprintf(errmsg, sizeof(errmsg), "WARNING: locale \"%s\", not supported.\n", locale_code);
+	    fprintf(stderr, "%s", errmsg);/* AUDIT biege: avoid format-bug warning */
     }
 #endif
 	
@@ -379,10 +383,22 @@ int main(int argc, char **argv)
     }
 #endif
 
+#ifndef HAVE_LIBFNV
+    if (set_nonsequential)
+      progerr("Hypermail isn't built with the libfnv hash library.\n"
+	     "You cannot use the nonsequential option.\n");
+#endif /* HAVE_LIBFNV */
+
     if (set_mbox && !strcasecmp(set_mbox, "NONE")) {
 	use_stdin = TRUE;
     }
 
+    /* the list of headers that we always show and that we want to avoid
+       showing twice when printing the body */
+    set_skip_headers = add_list(set_skip_headers, "from");
+    set_skip_headers = add_list(set_skip_headers, "date");
+    set_skip_headers = add_list(set_skip_headers, "subject");
+
     /*
      * Did they decide to use stdin by specifying it on the command line ?
      * If not then check and see if that is the default they wanted from
@@ -400,6 +416,19 @@ int main(int argc, char **argv)
 	set_mbox = NULL;
     }
 
+    /*
+    ** Deprecated options 
+    */
+    if (set_showhr) {
+      fprintf (stderr, "The \"showhr\" option has been deprecated. Ignoring it.\n");
+      set_showhr = FALSE;
+    }
+
+    if (set_usetable) {
+      fprintf (stderr, "The \"usetable\" option has been deprecated. Ignoring it.\n");
+      set_usetable = FALSE;
+    }
+
     /*
      * Read the contents of the file into the variables to be used
      * in printing out the pages.
@@ -407,6 +436,10 @@ int main(int argc, char **argv)
 
     ihtmlheaderfile = expand_contents(set_ihtmlheader);
     ihtmlfooterfile = expand_contents(set_ihtmlfooter);
+    ihtmlheadfile = expand_contents(set_ihtmlhead);
+    ihtmlhelpupfile = expand_contents(set_ihtmlhelpup);
+    ihtmlhelplowfile = expand_contents(set_ihtmlhelplow);
+    ihtmlnavbar2upfile = expand_contents(set_ihtmlnavbar2up);
     mhtmlheaderfile = expand_contents(set_mhtmlheader);
     mhtmlfooterfile = expand_contents(set_mhtmlfooter);
 
@@ -421,49 +454,35 @@ int main(int argc, char **argv)
 	set_dir = strreplace(set_dir, DIRNAME);
 
     if (!set_dir || !strcasecmp(set_dir, "NONE"))
-	set_dir = strreplace(set_dir, (strrchr(set_mbox, '/')) ?
-			     strrchr(set_mbox, '/') + 1 : set_mbox);
+		set_dir = strreplace(set_dir, (strrchr(set_mbox, '/')) ? strrchr(set_mbox, '/') + 1 : set_mbox);
 
     if (set_dir[strlen(set_dir) - 1] != PATH_SEPARATOR)
 	trio_asprintf(&set_dir, "%s%c", set_dir, PATH_SEPARATOR);
 
     if (!set_label || !strcasecmp(set_label, "NONE"))
-	set_label = set_mbox ?
-	    (strreplace(set_label, (strrchr(set_mbox, '/')) ?
-			strrchr(set_mbox, '/') + 1 : set_mbox)) : "stdin";
+		set_label = set_mbox ? (strreplace(set_label, (strrchr(set_mbox, '/')) ? strrchr(set_mbox, '/') + 1 : set_mbox)) : "stdin";
 
     /*
      * Which index file will be called "index.html"?
      */
 
-    index_name[1][DATE_INDEX] = setindex(set_defaultindex, "date",
-				      set_htmlsuffix);
-    index_name[1][THREAD_INDEX] = setindex(set_defaultindex, "thread",
-					set_htmlsuffix);
-    index_name[1][SUBJECT_INDEX] = setindex(set_defaultindex, "subject",
-					 set_htmlsuffix);
-    index_name[1][AUTHOR_INDEX] = setindex(set_defaultindex, "author",
-					set_htmlsuffix);
+	index_name[1][DATE_INDEX] = setindex(set_defaultindex, "date", set_htmlsuffix);
+	index_name[1][THREAD_INDEX] = setindex(set_defaultindex, "thread", set_htmlsuffix);
+	index_name[1][SUBJECT_INDEX] = setindex(set_defaultindex, "subject", set_htmlsuffix);
+	index_name[1][AUTHOR_INDEX] = setindex(set_defaultindex, "author", set_htmlsuffix);
     if (set_attachmentsindex) {
 	index_name[1][ATTACHMENT_INDEX]
 	    = setindex(set_defaultindex, "attachment", set_htmlsuffix);
     }
     if (set_folder_by_date || set_msgsperfolder) {
-	index_name[0][DATE_INDEX] = setindex(set_default_top_index, "date",
-					     set_htmlsuffix);
-	index_name[0][THREAD_INDEX] = setindex(set_default_top_index, "thread",
-					       set_htmlsuffix);
-	index_name[0][SUBJECT_INDEX] = setindex(set_default_top_index,
-						"subject", set_htmlsuffix);
-	index_name[0][AUTHOR_INDEX] = setindex(set_default_top_index, "author",
-					       set_htmlsuffix);
+		index_name[0][DATE_INDEX] = setindex(set_default_top_index, "date", set_htmlsuffix);
+		index_name[0][THREAD_INDEX] = setindex(set_default_top_index, "thread", set_htmlsuffix);
+		index_name[0][SUBJECT_INDEX] = setindex(set_default_top_index, "subject", set_htmlsuffix);
+		index_name[0][AUTHOR_INDEX] = setindex(set_default_top_index, "author", set_htmlsuffix);
 	if (set_attachmentsindex) {
-	    index_name[0][ATTACHMENT_INDEX] = setindex(set_default_top_index,
-						       "attachment",
-						       set_htmlsuffix);
+			index_name[0][ATTACHMENT_INDEX] = setindex(set_default_top_index, "attachment", set_htmlsuffix);
 	}
-	index_name[0][FOLDERS_INDEX] = setindex(set_default_top_index,
-						"folders", set_htmlsuffix);
+		index_name[0][FOLDERS_INDEX] = setindex(set_default_top_index, "folders", set_htmlsuffix);
     }
     else {
 	index_name[0][DATE_INDEX] = index_name[1][DATE_INDEX];
@@ -496,6 +515,13 @@ int main(int argc, char **argv)
     else
 	use_mailcommand = 1;
 
+#ifndef HAVE_LIBFNV
+    /* the nonsequential mode won't work unless we compiled the FNV hash library
+       (./configure --enable-libfnv) */
+    if (set_nonsequential)
+		progerr("the nonsequential mode is only available if you enabled the\n compilation" "of the fnv hash library. Try doing a\n\t./configure --enable-libfnv\n" "and recompile if you want to use this option.");
+#endif /* HAVE_LIBFNV */
+
     /* 
      * A little performance speed up.  The following was being done
      * over and over in the write functions. This way it is done once.
@@ -527,7 +553,7 @@ int main(int argc, char **argv)
     if (cmd_show_variables) {
 	if (!set_mbox)
 	    set_mbox = "NONE";
-	MakeConfig(TRUE);
+	MakeConfig(TRUE); 
 	free(configfile);
 	return (0);
     }
@@ -537,7 +563,7 @@ int main(int argc, char **argv)
     if (use_mbox && use_stdin) {
 	cmderr(lang[MSG_CANNOT_READ_FROM_BOTH_FILE_AND_STDIN]);
     }
-    if(set_append && use_mbox) {
+	if (set_append && use_mbox) {
         cmderr(lang[MSG_CANNOT_BOTH_READ_AND_WRITE_TO_MBOX]);
     }
 
@@ -563,19 +589,24 @@ int main(int argc, char **argv)
 	int num_displayable;
 	if (set_linkquotes)
 	    replylist = NULL;
+	/* we have to start with the msgnum - 1 so that the rest of the
+	   code works ok when there are no old headers. */
+	max_msgnum = set_startmsgnum - 1;
 	num_displayable = loadoldheaders(set_dir);
 	amount_old = max_msgnum + 1; /* counts gaps as messages */
 
 	/* start numbering at this number */
-	amount_new = num_displayable
-	  + parsemail(set_mbox, use_stdin, set_readone,
-		      set_increment, set_dir, set_inlinehtml, amount_old);
+		amount_new = num_displayable + parsemail(set_mbox, use_stdin, set_readone, set_increment, set_dir, set_inlinehtml, amount_old);
 	if (set_linkquotes)
 	    analyze_headers(max_msgnum + 1);
 
+	/* write the index of msgno/msgid_hash filenames */
+	if (set_nonsequential)
+		write_messageindex(0, max_msgnum + 1);
+
 	writearticles(amount_old, max_msgnum + 1);
 
-	if (set_delete_msgnum)
+	if (deletedlist)
 	    update_deletions(amount_old);
 
 	if (set_show_msg_links) {
@@ -588,16 +619,18 @@ int main(int argc, char **argv)
 	}
     }
     else {
-	amount_new =
-	    parsemail(set_mbox, use_stdin, set_readone, set_increment,
-		      set_dir, set_inlinehtml, 0);	/* number from 0 */
+	amount_new = parsemail(set_mbox, use_stdin, set_readone, set_increment, set_dir, 
+			       set_inlinehtml, set_startmsgnum);	/* number from 0 */
 	if (!matches_existing(0)) {
-	    fprintf(stderr, "Warning: first message in mailbox does not "
-		    "match first message in archive\n"
-		    "or obsolete gdbm file present.\n");
+		fprintf(stderr, "Warning: first message in mailbox does not " "match first message in archive\n" "or obsolete gdbm file present.\n");
 	}
 	if (set_linkquotes)
 	    analyze_headers(max_msgnum + 1);
+
+	/* write the index of msgno/msgid_hash filenames */
+	if (set_nonsequential)
+		write_messageindex(0, max_msgnum + 1);
+
 	writearticles(0, max_msgnum + 1);
     }
 
@@ -639,7 +672,7 @@ int main(int argc, char **argv)
 	    writeattachments(amount_new, NULL);
 	}
 	if (set_writehaof) 
-		writehaof(amount_new,NULL);
+            writehaof(amount_new, NULL);
 	if (set_folder_by_date || set_msgsperfolder)
 	    write_toplevel_indices(amount_new);
 	if (set_monthly_index || set_yearly_index)
@@ -654,18 +687,20 @@ int main(int argc, char **argv)
     if (set_uselock)
 	unlock_archive();
 
-#if 0
-    /* cleanup */
-    /* debug this before we activate it again (Daniel 1999-02-08) */
-    ConfigCleanup();
-#endif
-
     if (configfile)
 	free(configfile);
     if (ihtmlheaderfile)
 	free(ihtmlheaderfile);
     if (ihtmlfooterfile)
 	free(ihtmlfooterfile);
+    if (ihtmlheadfile)
+	free(ihtmlheadfile);
+    if (ihtmlhelpupfile)
+	free(ihtmlhelpupfile);
+    if (ihtmlhelplowfile)
+	free(ihtmlhelplowfile);
+    if (ihtmlnavbar2upfile)
+	free(ihtmlnavbar2upfile);
     if (mhtmlheaderfile)
 	free(mhtmlheaderfile);
     if (mhtmlfooterfile)
