@@ -28,7 +28,12 @@
 #define VAR
 #endif
 
+#ifdef __LCC__
+#include "../lcc/config.h"
+#else
 #include "../config.h"
+#endif
+
 #include "../patchlevel.h"
 
 #ifdef HAVE_UNISTD_H
@@ -77,6 +82,21 @@
 #define GDBM 1
 #endif
 
+#ifdef NO_MACRO
+#undef isspace
+#undef isdigit
+#undef isalpha
+#undef isupper
+#undef islower
+#undef isxdigit
+#undef isalnum
+#undef ispunct
+#undef isprint
+#undef isgraph
+#undef iscntrl
+#undef isascii
+#endif
+
 /*
 * this redefines the standard *printf() to use ours 
 */
@@ -147,7 +167,9 @@ typedef enum {
   FILTERED_DELETE = 1,
   FILTERED_EXPIRE = 2,
   FILTERED_OUT = 4,
-  FILTERED_REQUIRED = 8
+  FILTERED_REQUIRED = 8,
+  FILTERED_OLD = 16,
+  FILTERED_NEW = 32
 } filtered_reason_t;
 /* 
  * Path separator for attachment file path generation
@@ -248,9 +270,9 @@ struct emailinfo {
     struct reply *replylist;    /* list all possible direct replies to this */
     int isreply;
 #endif
-    struct emailsubdir *subdir;		/* NULL unless set_msgsperfolder or set_folder_by_date */
+    struct emailsubdir *subdir;	/* NULL unless set_msgsperfolder or set_folder_by_date */
     long exp_time;
-  int is_deleted;	/* 1=deleted, 2=expired, 4=filtered out, */
+    int is_deleted;	/* 1=deleted, 2=expired, 4=filtered out, */
 			/* 8=filtered (required line missing) */
 };
 
@@ -315,12 +337,18 @@ VAR char *top_index[NO_INDEX];
 VAR char *index_name[MAX_DIRECTORY_LEVELS][NO_INDEX];
 VAR int show_index[MAX_DIRECTORY_LEVELS][NO_INDEX];
 
+VAR char *antispamdomain;
+
 VAR int use_mailcommand;
 VAR int use_mailto;
 VAR int use_domainaddr;
 
 VAR char *ihtmlheaderfile;
 VAR char *ihtmlfooterfile;
+VAR char *ihtmlheadfile;
+VAR char *ihtmlhelpupfile;
+VAR char *ihtmlhelplowfile;
+VAR char *ihtmlnavbar2upfile;
 VAR char *mhtmlheaderfile;
 VAR char *mhtmlfooterfile;
 
@@ -328,6 +356,8 @@ VAR long firstdatenum;
 VAR long lastdatenum;
 VAR int max_msgnum;
 
+VAR char **msgnum_id_table;
+
 VAR const char *latest_folder_path;
 
 #ifdef MAIN_FILE
