@@ -4,7 +4,7 @@
 
 int ignorecontent(char *);
 int inlinecontent(char *);
-int preferedcontent(int *, char *);
+int preferedcontent(int *, char *, int);
 char *safe_filename(char *);
 char *getmaildate(char *);
 char *getfromdate(char *);
@@ -15,6 +15,7 @@ void print_progress(int, char *, char *);
 int parsemail(char *, int, int, int, char *, int, int);
 int parse_old_html(int, struct emailinfo *, int, int, struct reply **);
 int loadoldheaders(char *);
+int loadoldheadersfromGDBMindex(char *, int);
 void crossindex(void);
 void crossindexthread1(struct header *);
 void crossindexthread2(int);
