@@ -2,6 +2,7 @@
 #include "setup.h"
 #include "print.h"
 #include "printfile.h"
+#include "struct.h"
 
 /*
 ** printfile - print html header/footer file and fill in values 
@@ -73,7 +74,7 @@ int printfile(FILE *fp, char *format, char *label, char *subject,
 	    case 'A':		/* %e - email address of message author */
 		if (email && name) {
 		    fprintf(fp,
-			    "<meta name=\"Author\" content=\"%s (%s)\">",
+			    "<meta name=\"Author\" content=\"%s (%s)\" />",
 			    name, email);
 		}
 		continue;
@@ -97,7 +98,7 @@ int printfile(FILE *fp, char *format, char *label, char *subject,
 		if (charset && *charset) {
 		    /* only output this if we have a charset */
 		    fprintf(fp, "<meta http-equiv=\"Content-Type\""
-			    " content=\"text/html; charset=%s\">\n",
+			    " content=\"text/html; charset=%s\" />\n",
 			    charset);
 		}
 		continue;
@@ -105,7 +106,7 @@ int printfile(FILE *fp, char *format, char *label, char *subject,
 	    case 'D':		/* %D - date of message */
 		if (date) {
 		    fprintf(fp,
-			    "<meta name=\"Date\" content=\"%s\">",
+			    "<meta name=\"Date\" content=\"%s\" />",
 			    date);
 		}
 		continue;
@@ -150,13 +151,13 @@ int printfile(FILE *fp, char *format, char *label, char *subject,
 		    putc(*cp, fp);
 		continue;
 	    case 's':		/* %s - Subject of message or Index Title */
-		for (ptr = cp = convchars(subject); *cp; cp++)
+		for (ptr = cp = convchars(subject, charset); *cp; cp++)
 		    putc(*cp, fp);
 		free(ptr);
 		continue;
 	    case 'S':		/* %s - Subject of message or Index Title */
-		fprintf(fp, "<meta name=\"Subject\" content=\"%s\">",
-			cp = convchars(subject));
+		fprintf(fp, "<meta name=\"Subject\" content=\"%s\" />",
+			cp = convchars(subject, charset));
 		free(cp);
 		continue;
 	    case 't':
@@ -198,65 +199,118 @@ void print_main_header(FILE *fp, bool index_header, char *label, char *name,
     char *title;
     char *rp;
 
+    /* @@ JK: Don't know what to do with US-ASCII. If there's no charset,
+       assume the default one is ISO-8859-1 */
+    if (charset && *charset)
+      rp = charset;
+    else
+      rp = "ISO-8859-1";
     fprintf(fp,
-	    "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"\n"
-            "                      "
-	    "\"http://www.w3.org/TR/html4/strict.dtd\">\n");
-    fprintf(fp, "<html lang=\"%s\">\n", set_language);
+	    "<?xml version=\"1.0\" encoding=\"%s\"?>\n"
+	    "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n"
+	    "    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n",
+	    rp);
+    fprintf(fp, "<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"%s\">\n", set_language);
     fprintf(fp, "<head>\n");
 
     if (charset && *charset) {
 	/* charset info "as early as possible within the HEAD of the document"
 	 */
 	fprintf(fp, "<meta http-equiv=\"Content-Type\""
-		" content=\"text/html; charset=%s\">\n", charset);
+		" content=\"text/html; charset=%s\" />\n", charset);
     }
-    fprintf(fp, "<meta name=\"generator\" content=\"%s %s, see %s\">\n",
+    fprintf(fp, "<meta name=\"generator\" content=\"%s %s, see %s\" />\n",
                 PROGNAME, VERSION, HMURL);
 
     /* 
      * Strip off any trailing whitespace in TITLE so weblint is happy. 
      */
 
-    trio_asprintf(&title, "%s: %s", label, rp = convchars(subject));
+    trio_asprintf(&title, "%s: %s", label, rp = convchars(subject, charset));
     free(rp);
 
     rp = title + (strlen(title) - 1);
     while (isspace(*rp))
 	*rp-- = '\0';
 
+    /* 
+     * Assure the title meets HTML recommendations of no longer 
+     * than 64 characters. Truncate it if needed.
+     */
+
+    if (strlen(title) > 64)
+        *(title+64) = '\0';
+
     fprintf(fp, "<title>%s</title>\n", title);
     free(title);
 
     if (name && email)
-	fprintf(fp, "<meta name=\"Author\" content=\"%s (%s)\">\n",name,email);
-    fprintf(fp, "<meta name=\"Subject\" content=\"%s\">\n", rp =
-	    convchars(subject));
-	fprintf(fp, "<meta name=\"Date\" content=\"%s\">\n",date);
+	fprintf(fp, "<meta name=\"Author\" content=\"%s (%s)\" />\n",name,email);
+    fprintf(fp, "<meta name=\"Subject\" content=\"%s\" />\n", rp =
+	    convchars(subject, charset));
     free(rp);
+    if (date)
+	fprintf(fp, "<meta name=\"Date\" content=\"%s\" />\n",date);
     if (use_mailto)
-	fprintf(fp, "<link rev=\"made\" href=\"mailto:%s\">\n", set_mailto);
+	fprintf(fp, "<link rev=\"made\" href=\"mailto:%s\" />\n", set_mailto);
 
     /* print the css url according to the type of header */
     if (index_header && set_icss_url && *set_icss_url) {
-      fprintf(fp, "<link rel=\"stylesheet\" href=\"%s\" type=\"text/css\">\n",
+      fprintf(fp, "<link rel=\"stylesheet\" href=\"%s\" type=\"text/css\" />\n",
               set_icss_url);
 
     } else if (!index_header && set_mcss_url && *set_mcss_url) {
-      fprintf(fp, "<link rel=\"stylesheet\" href=\"%s\" type=\"text/css\">\n",
+      fprintf(fp, "<link rel=\"stylesheet\" href=\"%s\" type=\"text/css\" />\n",
               set_mcss_url);
 
     } else {
       /*
        * if style sheets are not specified, emit a default one.
        */
+      /* @@ JK: the old css */
+#if 0
       fprintf(fp, "<style type=\"text/css\">\n");
       fprintf(fp, "body {color: black; background: #ffffff}\n");
       fprintf(fp, "h1.center {text-align: center}\n");
       fprintf(fp, "div.center {text-align: center}\n");
+      fprintf(fp, ".quotelev1 {color : #990099}\n");
+      fprintf(fp, ".quotelev2 {color : #ff7700}\n");
+      fprintf(fp, ".quotelev3 {color : #007799}\n");
+      fprintf(fp, ".quotelev4 {color : #95c500}\n");
+      fprintf(fp, ".headers {background : #e0e0d0}\n");
+      fprintf(fp, ".links {background : #f8f8e0}\n");
       fprintf(fp, "</style>\n");
+#endif
+      /* @@ JK: the new css */
+      fprintf (fp, "<style type=\"text/css\">\n");
+      
+      fprintf (fp,"/*<![CDATA[*/\n");
+      fprintf (fp, "/* To be incorporated in the main stylesheet, don't code it in hypermail! */\n");
+      fprintf (fp, "body {color: black; background: #ffffff}\n");
+      fprintf (fp, "dfn {font-weight: bold;}\n");
+      fprintf (fp, "pre { background-color:inherit;}\n");
+      fprintf (fp, ".head { border-bottom:1px solid black;}\n");
+      fprintf (fp, ".foot { border-top:1px solid black;}\n");
+      fprintf (fp, "th {font-style:italic;}\n");
+      fprintf (fp, "table { margin-left:2em;}");
+
+      /* JK: This was the WAI rule before */
+      /* fprintf (fp, "#body {background-color:#fff;}\n"); */
+      fprintf (fp, "map ul {list-style:none;}\n");
+      fprintf (fp, "#mid { font-size:0.9em;}\n");
+      fprintf (fp, "#received { float:right;}\n");
+      fprintf (fp, "address { font-style:inherit ;}\n");
+      fprintf (fp, "/*]]>*/\n");
+      fprintf(fp, ".quotelev1 {color : #990099}\n");
+      fprintf(fp, ".quotelev2 {color : #ff7700}\n");
+      fprintf(fp, ".quotelev3 {color : #007799}\n");
+      fprintf(fp, ".quotelev4 {color : #95c500}\n");
+      fprintf (fp, "</style>\n");
     }
 
+    if (ihtmlheadfile)
+      fprintf (fp, "%s", ihtmlheadfile);
+
     fprintf(fp, "</head>\n");
     fprintf(fp, "<body>\n");
 }
@@ -269,25 +323,12 @@ void print_msg_header(FILE *fp, char *label, char *subject,
 		      char *dir, char *name, char *email, char *msgid,
 		      char *charset, time_t date, char *filename)
 {
-    char *ptr;
     if (mhtmlheaderfile)
 	printfile(fp, mhtmlheaderfile, set_label, subject, set_dir, name, 
 		  email, msgid, charset, secs_to_iso_meta(date), filename);
     else {
 	print_main_header(fp, FALSE, set_label, name, email, subject,
 			  charset, secs_to_iso_meta(date), filename);
-#if 0 /* JK modified this */       
-	fprintf(fp, "<h1 class=\"center\">%s</h1>\n",
-		ptr = convchars(subject));
-	free(ptr);
-#endif
-	fprintf(fp, "<h1>%s</h1>\n",
-		ptr = convchars(subject));
-	free(ptr);
-#if 0 /* JK: and removed this as it looked a bit strange */
-	if (!set_usetable)
-	    fprintf(fp, "<hr>\n<p>\n");
-#endif
     }
 }
 
@@ -302,12 +343,15 @@ void print_index_header(FILE *fp, char *label, char *dir, char *subject,
 	printfile(fp, ihtmlheaderfile, label, subject, dir, NULL, NULL,
 		  NULL, NULL, NULL, filename);
     else {
+	/* print the navigation bar to upper levels */
 	print_main_header(fp, TRUE, label, NULL, NULL, subject, NULL, NULL, NULL);
-	fprintf(fp, "<h1 class=\"center\">%s<br>%s</h1>\n", label, subject);
-#if 0 /*@@ JK: removed it */	
-	if (!set_usetable)
-	    fprintf(fp, "<hr>\n");
-#endif
+	
+	fprintf (fp, "<div class=\"head\">\n");
+	if (ihtmlnavbar2upfile)
+	  fprintf(fp, "<map title=\"%s\" id=\"upper\">\n%s</map>\n", 
+		  lang[MSG_NAVBAR2UPPERLEVELS], ihtmlnavbar2upfile);
+
+	fprintf(fp, "<h1>%s %s</h1>\n", label, subject);
     }
 }
 
@@ -316,7 +360,7 @@ void print_index_header(FILE *fp, char *label, char *dir, char *subject,
 */
 
 void printfooter(FILE *fp, char *htmlfooter, char *label, char *dir,
-		 char *subject, char *filename)
+		 char *subject, char *filename, bool close_div)
 {
     printcomment(fp, "trailer", "footer");
 
@@ -324,13 +368,27 @@ void printfooter(FILE *fp, char *htmlfooter, char *label, char *dir,
 	printfile(fp, htmlfooter, label, subject,
 		  dir, NULL, NULL, NULL, NULL, NULL, filename);
     else {
-	if (set_showhr && !set_usetable)
-	    fprintf(fp, "<hr>\n");
 	fprintf(fp, "<p><small><em>\n");
 	fprintf(fp, "%s ", lang[MSG_ARCHIVE_GENERATED_BY]);
 	fprintf(fp, "<a href=\"%s\">%s %s</a> \n", HMURL, PROGNAME, VERSION);
 	fprintf(fp, ": %s\n", getlocaltime());
 	fprintf(fp, "</em></small></p>\n");
+	if (close_div)
+	  fprintf (fp, "</div>\n");
 	fprintf(fp, "</body>\n</html>\n");
     }
 }
+
+/*
+** Prints the HTML last message and last archived date (used in the indexes).
+*/
+
+void printlaststats (FILE *fp, long lastdatenum)
+{
+    fprintf (fp, "<ul>\n");
+    fprintf (fp, "<li><dfn><a id=\"end\" name=\"end\">%s</a></dfn>: <em>%s</em></li>\n",
+	     lang[MSG_LAST_MESSAGE_DATE], getdatestr(lastdatenum));
+
+    fprintf (fp, "<li><dfn>%s</dfn>: %s</li>\n",  lang[MSG_ARCHIVED_ON], getlocaltime());
+    fprintf (fp, "</ul>\n");
+}
