@@ -7,7 +7,7 @@
 ** Date functions - date.c
 */
 
-long convtoyearsecs(char *);
+time_t convtoyearsecs(char *);
 char *getlocaltime(void);
 void gettimezone(void);
 void getthisyear(void);
@@ -43,13 +43,19 @@ int find_max_msgnum(void);
 int is_empty_archive(void);
 void symlink_latest(void);
 struct emailsubdir *msg_subdir(int, time_t);
-char *msg_href(struct emailinfo *, struct emailinfo *);
+char *msg_href(struct emailinfo *, struct emailinfo *, int);
 char *msg_relpath(struct emailinfo *, struct emailinfo *);
 char *articlehtmlfilename(struct emailinfo *);
 char *htmlfilename(const char *, struct emailinfo *, const char *);
 char *haofname(struct emailinfo *);
 int matches_existing(int);
 
+char *messageindex_name(void);
+int find_max_msgnum_id(void);
+char **read_msgnum_id_table(int);
+void free_msgnum_id_table(char **, int);
+char *message_name(struct emailinfo *);
+
 /*
 ** hypermail.c functions
 */
@@ -88,10 +94,11 @@ char *PushByte(struct Push *, char);
 char *PushString(struct Push *, const char *);
 char *PushNString(struct Push *, const char *, int);
 
+char *strcasestr (char *, const char *);
 char *strsav(const char *);
 char *strreplace(char *, char *);
 void strcpymax(char *, const char *, int);
-char *strcasestr(char *, const char *);
+void strtolower (char *);
 char *stripzone(char *);
 int numstrchr(char *, char);
 char *getvalue(char *);
@@ -101,12 +108,14 @@ char *oneunre(char *);
 int isquote(const char *);
 char *replace(char *, char *, char *);
 char *replacechar(char *, char, char *);
-char *convchars(char *);
+char *convdash(char *);
+char *convchars(char *, char *);
+char *convcharsnospamprotect(char *, char *);
 char *unconvchars(char *);
 char *makemailcommand(char *, char *, char *, char *);
 char *unspamify(char *);
 char *parseemail(char *, char *, char *);
-char *parseurl(char *);
+char *parseurl(char *, char *);
 
 char *hm_strchr(const char *, int);
 void iso2022_state(const char *str, int *state, int *esc);
@@ -120,12 +129,16 @@ char *unquote(char *line);
 char *remove_hypermail_tags(char *line);
 int compute_quoted_percent(struct body *bp);
 int is_sig_start(const char *line);
+int find_quote_depth(char *);
+char *find_quote_class(char *);
 
+#ifdef NOTDEF
 #ifdef lint
 int isspace(int);
 int isalpha(int);
 int isalnum(int);
 int isxdigit(int);
 #endif
+#endif
 
 #endif
