@@ -34,6 +34,7 @@ void MakeConfig(bool);		/* set to TRUE to include comments */
 void PreConfig(void);
 void PostConfig(void);
 
+extern char *set_antispam_at;
 extern char *set_language;
 extern char *set_htmlsuffix;
 extern char *set_mbox;
@@ -50,6 +51,7 @@ extern bool set_inlinehtml;
 extern bool set_increment;
 extern bool set_readone;
 extern bool set_reverse;
+extern bool set_reverse_folders;
 extern bool set_showprogress;
 extern bool set_showheaders;
 extern bool set_showbr;
@@ -78,7 +80,10 @@ extern bool set_usegdbm;
 extern bool set_writehaof;
 extern bool set_append;
 extern char *set_append_filename;
+extern bool set_nonsequential;
 extern bool set_warn_surpressions;
+extern bool set_files_by_thread;
+extern bool set_href_detection;
 
 extern int set_showhtml;
 extern int set_thrdlevels;
@@ -88,6 +93,7 @@ extern int set_locktime;
 extern int set_searchbackmsgnum;
 extern int set_quote_hide_threshold;
 extern int set_thread_file_depth;
+extern int set_startmsgnum;
 
 extern int set_save_alts;
 extern char *set_alts_text;
@@ -115,6 +121,7 @@ extern struct hmlist *set_inline_types;
 extern struct hmlist *set_prefered_types;
 extern struct hmlist *set_ignore_types;
 extern struct hmlist *set_show_headers;
+extern struct hmlist *set_skip_headers;
 extern struct hmlist *set_avoid_indices;
 extern struct hmlist *set_avoid_top_indices;
 extern struct hmlist *set_filter_out;
@@ -124,6 +131,10 @@ extern struct hmlist *set_filter_require_full_body;
 
 extern char *set_ihtmlheader;
 extern char *set_ihtmlfooter;
+extern char *set_ihtmlhead;
+extern char *set_ihtmlhelpup;
+extern char *set_ihtmlhelplow;
+extern char *set_ihtmlnavbar2up;
 extern char *set_mhtmlheader;
 extern char *set_mhtmlfooter;
 
@@ -131,6 +142,8 @@ extern char *set_attachmentlink;
 extern char *set_unsafe_chars;
 extern bool set_linkquotes;
 
+extern char *set_antispamdomain;
+
 extern char *set_folder_by_date;
 extern char *set_latest_folder;
 extern char *set_base_url;
@@ -142,6 +155,8 @@ extern bool set_iso2022jp;
 extern struct hmlist *set_deleted;
 extern struct hmlist *set_expires;
 extern struct hmlist *set_delete_msgnum;
+extern char *set_delete_older;
+extern char *set_delete_newer;
 extern int set_delete_level;
 /* values for delete_level: */
 enum { DELETE_REMOVES_FILES,
