@@ -26,7 +26,7 @@
 #include "hypermail.h"
 #include "setup.h"
 #include "parse.h"
-
+#include "uconvert.h"
 
 /*
 ** Push byte onto a buffer realloc the buffer if needed.
@@ -197,6 +197,25 @@ void strcpymax(char *dest, const char *src, int n)
     *dest = 0;
 }
 
+/*
+** strtolower - converts a string to lower case.
+*/
+
+void strtolower(char *string)
+{
+  char *ptr;
+
+  if (!string)
+    return;
+
+  ptr = string;
+  while (*ptr) {
+    if (isalpha (*ptr))
+      *ptr = tolower (*ptr);
+    ptr++;
+  }
+}
+
 #ifndef HAVE_STRCASESTR
 /*
 ** strcasestr() - case insensitive strstr()
@@ -488,6 +507,31 @@ int isquote(const char *line)
     return 0;
 }
 
+/*
+** A ripoff from convchars, for a special case.
+** Converts - to &ndash;. Used for storing strings
+** in XML comments.
+**
+** Returns an ALLOCATED string!
+*/
+char *convdash (char *line)
+{
+    struct Push buff;
+
+    INIT_PUSH(buff);		/* init macro */
+
+    /* avoid strlen() for speed */
+
+    for (; *line; line++) {
+
+      if (*line == '-')
+	PushString(&buff, "&ndash;");
+      else
+	PushByte(&buff, *line);
+    }
+    RETURN_PUSH(buff);
+} /* end convdash() */
+
 /*
 ** Converts <, >, and & to &lt;, &gt; and &amp;.
 ** It was ugly. Now its better. And probably faster.
@@ -495,10 +539,17 @@ int isquote(const char *line)
 ** Returns an ALLOCATED string!
 */
 
-char *convchars(char *line)
+char *convcharsreal(char *line, char *charset, int spamprotect)
 {
     struct Push buff;
     int in_ascii = TRUE, esclen = 0;
+    int seen_at = FALSE;
+    bool is_iso_8859_1;
+
+    if (charset && !strcasecmp ("iso-8859-1", charset))
+      is_iso_8859_1 = TRUE;
+    else
+      is_iso_8859_1 = FALSE;
 
     INIT_PUSH(buff);		/* init macro */
 
@@ -518,6 +569,20 @@ char *convchars(char *line)
 		}
 	}
 
+	/* @@ JK : try to convert from the WinLatin1 code */
+	if (is_iso_8859_1
+	    && (unsigned char) (*line) >= 0x80 && (unsigned char) (*line) <= 0x9f) {
+	  char *unicode_entity;
+	  trio_asprintf (&unicode_entity, 
+			 "&#x%x;", WIN1252CP[(unsigned char) (*line) - WIN1252CP_length]);
+
+	  if (unicode_entity) {
+	    PushString(&buff, (const char *) unicode_entity);
+	    free (unicode_entity);
+	  }
+	  continue;
+	}
+
 	switch (*line) {
 	case '<':
 	    PushString(&buff, "&lt;");
@@ -531,6 +596,18 @@ char *convchars(char *line)
 	case '\"':
 	    PushString(&buff, "&quot;");
 	    break;
+	case '@': /* pkn added: simple "antispam" measure */
+	    PushString(&buff, "&#64;");
+	    seen_at = TRUE;
+	    break;
+	case '.': /* pkn added */
+	    if (seen_at && spamprotect)
+	    {
+	    	PushString(&buff, "&#46;<!--nospam-->");
+	    	seen_at = FALSE;
+	    	break;
+	    }
+	    /* fall through */
 	default:
 	    PushByte(&buff, *line);
 	}
@@ -538,6 +615,42 @@ char *convchars(char *line)
     RETURN_PUSH(buff);
 } /* end convchars() */
 
+char *convcharsnospamprotect(char *line, char *charset)
+{
+    return convcharsreal(line, charset, FALSE);
+}
+char *convchars(char *line, char *charset)
+{
+    return convcharsreal(line, charset, set_spamprotect);
+}
+
+/*
+** Converts from an Unicode entity to the equivalent winlatin charset
+*/
+static bool unconvwinlatin1 (char *entity, char *conv_char)
+{
+  int i;
+  int value = 0;
+
+  if (!entity || *entity == '\0' || *entity != '#')
+    return FALSE;
+
+  if (sscanf (entity, "#x%x;", &value) != 1 || value == 0)
+    return FALSE;
+
+  for (i = 0; i <  WIN1252CP_length; i++) {
+    if (WIN1252CP[i] == value)
+      break;
+  }
+
+  if (i == WIN1252CP_length)
+    return FALSE;
+
+  *conv_char =  (char) (i + 0x80);
+
+  return TRUE;
+} /* end unconvwinlatin1 */
+
 /*
 ** Just the opposite of convchars().
 ** Completely rewritten 17th Nov 1998 by Daniel.
@@ -545,6 +658,7 @@ char *convchars(char *line)
 
 char *unconvchars(char *line)
 {
+    char conv_char;
     struct Push buff;
     INIT_PUSH(buff);
 
@@ -562,15 +676,30 @@ char *unconvchars(char *line)
 		PushByte(&buff, '&');
 		line += 4;
 	    }
+	    else if (!strncmp("&#46;<!--nospam-->", line + 1, 18)) {
+	        PushByte(&buff, '.');
+	        line += 18;
+	    }
 	    else if (!strncmp("quot;", line + 1, 5)) {
 		PushByte(&buff, '\"');
 		line += 5;
 	    }
+	    else if (!strncmp("#64;", line + 1, 4)) {
+		PushByte(&buff, '@');
+		line += 4;
+	    }
+	    else if (!strncmp("ndash;", line + 1, 6)) {
+	        PushByte(&buff, '-');
+		line += 6;
+	    }
+	    else if (unconvwinlatin1 (line + 1, &conv_char)) {
+		PushByte(&buff, conv_char);
+	    }
 	    else
-		PushByte(&buff, *line);
+	        PushByte(&buff, *line);
 	}
 	else
-	    PushByte(&buff, *line);
+	  PushByte(&buff, *line);
     }
     RETURN_PUSH(buff);
 }
@@ -622,14 +751,33 @@ static char *translateurl(char *url)
 {
     char *p;
     struct Push buff;
-    int in_ascii = TRUE, esclen = 0;
+    unsigned int i;
+    char hexa[5];
+
     INIT_PUSH(buff);
 
     for (p = url; *p; p++) {
 	if (*p == '&')
 	    PushString(&buff, "&amp;");
-	else
-	    PushByte(&buff, *p);
+	else if (*p == '@' && set_mailcommand) 	/* we need to code @ as such to
+						   avoid a misconvertion */
+	  PushString(&buff, "&#64;");
+	else if ((unsigned char) *p > 127 && set_mailcommand) {
+	  i = (unsigned char) *p & 0xFF;
+	  snprintf (hexa, sizeof (hexa) -1, "%%%02x", i);
+	  PushString (&buff, hexa);
+	}
+	else {
+	  /* JK: reaching this point means that we didn't need to do any of the
+	     special conversions for URLs, but we still need to make
+	     sure that it's a valid character. */
+	  char s[2] = "a";
+	  char *ptr;
+	  s[0] = *p;
+	  ptr = convchars (s, NULL);
+	  PushString (&buff, ptr);
+	  free (ptr);
+	}
     }
     RETURN_PUSH(buff);
 }
@@ -733,7 +881,7 @@ char *makemailcommand(char *mailcommand, char *email, char *id, char *subject)
     else
 	hasre = 0;
 
-    convsubj = convchars(subject);
+    convsubj = convchars(subject, NULL);
 
     /* remade to deal with any-length strings */
     /* tmpsubject = maprintf("%s%s", (hasre) ? "" : "Re: ", (convsubj) ? convsubj : ""); */
@@ -760,7 +908,9 @@ char *makemailcommand(char *mailcommand, char *email, char *id, char *subject)
 	newcmd2 = replacechar(newcmd, '+', "%2B");
 	free(newcmd);
 
-	newcmd = newcmd2;	/* this is the new string */
+	/* escape the special characters following the URL convention */
+	newcmd = translateurl (newcmd2);
+	free (newcmd2);
 
 	free(tmpsubject);
 
@@ -773,12 +923,12 @@ char *unspamify(char *s)
     char *p;
     if (!s)
 	return s;
-    if (!strchr(s, '@') && ((p = strstr(s, "_at_")) != NULL)) {
+    if (!strchr(s, '@') && ((p = strstr(s, set_antispam_at)) != NULL)) {
 	struct Push buff;
 	INIT_PUSH(buff);
 	PushNString(&buff, s, p - s);
 	PushByte(&buff, '@');
-	PushString(&buff, p + 4);
+	PushString(&buff, p + strlen(set_antispam_at));
 	return PUSH_STRING(buff);
     }
     return strsav(s);
@@ -816,7 +966,7 @@ char *parseemail(char *input,	/* string to parse */
     int in_ascii = TRUE, esclen = 0;
 
     if(set_spamprotect)
-      at="_at_";
+      at=set_antispam_at;
     else
       at="@";
 
@@ -919,7 +1069,9 @@ static char *url[] = {
     "https://",
     "news:",
     "ftp://",
-    "file://",
+#if 0
+    "file://",/* can expose private files outside the archive in some cases? */
+#endif
     "gopher://",
     "nntp://",
     "wais://",
@@ -929,7 +1081,7 @@ static char *url[] = {
     NULL
 };
 
-char *parseurl(char *input)
+char *parseurl(char *input, char *charset)
 {
     struct Push buff;		/* output buffer */
     char urlbuff[256];
@@ -949,7 +1101,7 @@ char *parseurl(char *input)
 	 * have any; it will be a big win for average input if we follow
 	 * this trivial heuristic. 
 	 */
-	return convchars(input);
+	return convchars(input, charset);
     }
     INIT_PUSH(buff);
 
@@ -1029,8 +1181,8 @@ char *parseurl(char *input)
 	    if (set_iso2022jp)
 		    urlscan = sscanf(inputp, "%255[^] \033)>\"\'\n[\t\\]", urlbuff);
 	    else
-		    urlscan = sscanf(inputp, "%255[^] )>\"\'\n[\t\\]", urlbuff);
-	    if (urlscan) {
+		    urlscan = sscanf(inputp, "%255[^] )<>\"\'\n[\t\\]", urlbuff);
+	    if (urlscan == 1) {
 	        char *r;
 	
 		/* 
@@ -1063,7 +1215,7 @@ char *parseurl(char *input)
 			      "<a href=\"%s%s\">%s%s</a>",
 			      thisprotocol, urlbuff2, thisprotocol, urlbuff2);
 		PushString(&buff, tempbuff); /* append the tag buffer */
-		inputp += strlen(urlbuff2);
+		inputp += strlen(urlbuff);
 		free(urlbuff2);
 	    } else {
 	        PushString(&buff, thisprotocol);
@@ -1169,3 +1321,24 @@ hm_strchr(const char *str, int ch)
 		return((char *)NULL);
 	}
 }
+
+#if ! HAVE_STRERROR
+/*************************************************
+*     Provide strerror() for non-ANSI libraries  *
+*************************************************/
+
+/* Some old-fashioned systems still around (e.g. SunOS4) don't have strerror()
+in their libraries, but can provide the same facility by this simple
+alternative function. */
+
+extern int   sys_nerr;
+extern char *sys_errlist[];
+
+char *
+strerror(int n)
+{
+if (n < 0 || n >= sys_nerr) return "unknown error number";
+return sys_errlist[n];
+}
+#endif /* HAVE_STRERROR */
+
