@@ -30,8 +30,9 @@ int rbs_bigtime = 0;
 #include "struct.h"
 #include "parse.h"
 
-#define HAVE_PCRE
-#ifdef HAVE_PCRE
+#ifdef __LCC__
+#include "../lcc/pcre.h"
+#else
 #include <pcre.h>
 #endif
 
@@ -54,8 +55,7 @@ unsigned hash(char *s)
     return (hashval % HASHSIZE);
 }
 
-void fill_email_dates(struct emailinfo *e, char *date, char *fromdate,
-		      char *isodate, char *isofromdate)
+void fill_email_dates(struct emailinfo *e, char *date, char *fromdate, char *isodate, char *isofromdate)
 {
     bool fromdate_valid = 1, date_valid = 1;
     /* fromdate may be empty (zero length string rather than NULL)
@@ -69,6 +69,9 @@ void fill_email_dates(struct emailinfo *e, char *date, char *fromdate,
 	e->datestr = strsav(date);
     }
     else {
+	e->date = e->fromdate = -1;
+	if (date == NULL)
+	    date = NODATE;
 	if (!strcmp(date, NODATE)) {
 	    date_valid = 0;
 	}
@@ -85,9 +88,7 @@ void fill_email_dates(struct emailinfo *e, char *date, char *fromdate,
 	}
 	if (fromdate_valid && !date_valid) {
 #ifdef PH_DATE_DEBUG
-	    fprintf(stderr,
-		    "%d: %s: using fromdate '%s' for both (date '%s')\n",
-		    num, msgid, fromdate, date);
+			fprintf(stderr, "%d: %s: using fromdate '%s' for both (date '%s')\n", num, msgid, fromdate, date);
 #endif
 	    e->fromdatestr = strsav(fromdate);
 	    e->datestr = strsav(fromdate);
@@ -95,9 +96,7 @@ void fill_email_dates(struct emailinfo *e, char *date, char *fromdate,
 	}
 	else if (!fromdate_valid && date_valid) {
 #ifdef PH_DATE_DEBUG
-	    fprintf(stderr,
-		    "%d: %s: using date '%s' for both (fromdate '%s')\n",
-		    num, msgid, date, fromdate);
+			fprintf(stderr, "%d: %s: using date '%s' for both (fromdate '%s')\n", num, msgid, date, fromdate);
 #endif
 	    e->fromdatestr = strsav(date);
 	    e->datestr = strsav(date);
@@ -105,9 +104,7 @@ void fill_email_dates(struct emailinfo *e, char *date, char *fromdate,
 	}
 	else if (!fromdate_valid && !date_valid) {
 #ifdef PH_DATE_DEBUG
-	    fprintf(stderr,
-		    "%d: %s: fromdate '%s' and date '%s' both bad\n", num,
-		    msgid, fromdate, date);
+			fprintf(stderr, "%d: %s: fromdate '%s' and date '%s' both bad\n", num, msgid, fromdate, date);
 #endif
 	    e->fromdatestr = strsav(fromdate);
 	    e->datestr = strsav(date);
@@ -126,10 +123,7 @@ void fill_email_dates(struct emailinfo *e, char *date, char *fromdate,
 		    date_parsed[24] = '\0';
 		    fromdate_parsed[24] = '\0';
 		    rbs_bigtime++;
-		    fprintf(stderr,
-			    "%d [%d:%d]: %s: received before sent\n %-38.38s %-30.38s\n %-38.38s %-38.38s\n",
-			    num, rbs, rbs_bigtime, msgid, fromdate, date,
-			    fromdate_parsed, date_parsed);
+					fprintf(stderr, "%d [%d:%d]: %s: received before sent\n %-38.38s %-30.38s\n %-38.38s %-38.38s\n", num, rbs, rbs_bigtime, msgid, fromdate, date, fromdate_parsed, date_parsed);
 		}
 	    }
 #endif
@@ -144,10 +138,7 @@ void fill_email_dates(struct emailinfo *e, char *date, char *fromdate,
 ** handily looked up and retrieved using any of these criteria.
 */
 
-struct emailinfo *addhash(int num, char *date, char *name, 
-		char *email, char *msgid, char *subject, 
-		char *inreply, char *fromdate, char *charset, 
-		char *isodate, char *isofromdate, struct body *sp)
+struct emailinfo *addhash(int num, char *date, char *name, char *email, char *msgid, char *subject, char *inreply, char *fromdate, char *charset, char *isodate, char *isofromdate, struct body *sp)
 {
     struct emailinfo *e;
     struct hashemail *h;
@@ -163,8 +154,7 @@ struct emailinfo *addhash(int num, char *date, char *name,
 	/* SEVERE ERROR, all mails MUST have a Message-ID, ignore it! */
 	if (set_require_msgids) {
 	    if (set_warn_surpressions)
-	        printf("Message-ID is missing, ignoring message with subject '%s'.\n",
-		       subject ? subject : "<unknown>");
+				printf("Message-ID is missing, ignoring message with subject '%s'.\n", subject ? subject : "<unknown>");
 	    return NULL;
 	}
 	else {
@@ -184,8 +174,9 @@ struct emailinfo *addhash(int num, char *date, char *name,
 	}
     }
 
-    if ((msgid_dup && set_discard_dup_msgids) && set_showprogress) {
-	printf ("%d: Message-ID collision, '%s' already present - discarding message.\n", num, msgid);
+    if (msgid_dup && set_discard_dup_msgids) {
+	if (set_showprogress)
+		printf("%d: Message-ID collision, '%s' already present - discarding message.\n", num, msgid);
 	return NULL;
     }
 
@@ -199,8 +190,7 @@ struct emailinfo *addhash(int num, char *date, char *name,
 	newmsgid = emalloc(MSGDSTRLEN);
 	do {
 	    msgid_dup = 0;
-	    sprintf(newmsgid, "%d.%4.4d@hypermail.dummy", time(NULL),
-		    freedummy);
+			sprintf(newmsgid, "%d.%4.4d@hypermail.dummy", time(NULL), freedummy);
 	    h = (struct hashemail *)etable[hash(newmsgid)];
 	    while (h) {
 		if (!strcmp(h->data->msgid, newmsgid)) {
@@ -217,8 +207,7 @@ struct emailinfo *addhash(int num, char *date, char *name,
 	        if (msgid_missing) 
 		    printf("\n%d Message-ID missing, using %s instead.\n", num, newmsgid);
 	        else
-		    printf("\n%d Message-ID collision, '%s' already present - using %s instead.\n", 
-                           num, msgid, newmsgid);
+					printf("\n%d Message-ID collision, '%s' already present - using %s instead.\n", num, msgid, newmsgid);
             }
 	    msgid = newmsgid;
 	}
@@ -227,7 +216,7 @@ struct emailinfo *addhash(int num, char *date, char *name,
 	        if (msgid_missing)
 		    printf("\n%d Message-ID missing, failed to find free id - dropping message.\n", num);
 	        else
-		    printf ("\n%d Message-ID collision, '%s' already present - failed to find free id - dropping message.\n", num, msgid);
+					printf("\n%d Message-ID collision, '%s' already present - failed to find free id - dropping message.\n", num, msgid);
             }
 	    return NULL;
 	}
@@ -305,7 +294,7 @@ struct emailinfo *addhash(int num, char *date, char *name,
     return e;			/* the actual mail struct pointer */
 }
 
-int insert_in_lists(struct emailinfo *emp, const bool *require_filter, int rlen)
+int insert_in_lists(struct emailinfo *emp, const bool * require_filter, int rlen)
 {
     int i;
     if (set_delete_msgnum) {
@@ -313,13 +302,13 @@ int insert_in_lists(struct emailinfo *emp, const bool *require_filter, int rlen)
 	sprintf(num_str, "%d", emp->msgnum);
 	if (inlist(set_delete_msgnum, num_str)) {
 	    if (!emp->is_deleted) {
-	        if(emp->subdir)
+				if (emp->subdir)
 		    --emp->subdir->count;
 	    }
 	    emp->is_deleted = FILTERED_DELETE;
 	}
     }
-    for(i = 0; i < rlen; ++i) {
+	for (i = 0; i < rlen; ++i) {
 	if (!require_filter[i]) {
 	    emp->is_deleted = FILTERED_REQUIRED;
 	}
@@ -333,15 +322,27 @@ int insert_in_lists(struct emailinfo *emp, const bool *require_filter, int rlen)
 	deletedlist = h;
 	if (set_warn_surpressions && require_filter) {
 	    const char *option = "??";
-	    switch(emp->is_deleted)
-	    {
-	    case FILTERED_DELETE:   option = "deleted or delete_msgnum"; break;
-	    case FILTERED_EXPIRE:   option = "expires"; break;
-	    case FILTERED_OUT:      option = "filter_out or filter_out_full_body"; break;
-	    case FILTERED_REQUIRED: option = "filter_require or filter_require_full_body"; break;
+			switch (emp->is_deleted) {
+			case FILTERED_DELETE:
+				option = "deleted or delete_msgnum";
+				break;
+			case FILTERED_EXPIRE:
+				option = "expires";
+				break;
+			case FILTERED_OUT:
+				option = "filter_out or filter_out_full_body";
+				break;
+			case FILTERED_REQUIRED:
+				option = "filter_require or filter_require_full_body";
+				break;
+			case FILTERED_OLD:
+				option = "delete_older";
+				break;
+			case FILTERED_NEW:
+				option = "delete_newer";
+				break;
 	    }
-	    printf("message %d deleted under option %s. msgid: %s\n",
-		   emp->msgnum+1, option, emp->msgid);
+			printf("message %d deleted under option %s. msgid: %s\n", emp->msgnum + 1, option, emp->msgid);
 	}
     }
     else {
@@ -365,8 +366,7 @@ int insert_in_lists(struct emailinfo *emp, const bool *require_filter, int rlen)
 */
 #ifdef USE_OBSOLETE_HASHREPLYLOOKUP
 
-struct emailinfo *hashreplylookup(int msgnum, char *inreply,
-				  int *issubjmatch)
+struct emailinfo *hashreplylookup(int msgnum, char *inreply, int *issubjmatch)
 {
     struct hashemail *ep;
 
@@ -376,8 +376,7 @@ struct emailinfo *hashreplylookup(int msgnum, char *inreply,
     *issubjmatch = 0;
     ep = etable[hash(inreply)];
     while (ep) {
-	if ((msgnum != ep->data->msgnum) &&
-	    (strcmp(inreply, ep->data->msgid) == 0)) {
+		if ((msgnum != ep->data->msgnum) && (strcmp(inreply, ep->data->msgid) == 0)) {
 #if DEBUG_THREAD
 	    fprintf(stderr, "match on msgid [%d]\n", ep->data->msgnum);
 #endif
@@ -388,8 +387,7 @@ struct emailinfo *hashreplylookup(int msgnum, char *inreply,
 
     ep = etable[hash(inreply)];
     while (ep != NULL) {
-	if ((msgnum != ep->data->msgnum) &&
-	    (strstr(inreply, ep->data->datestr))) {
+		if ((msgnum != ep->data->msgnum) && (strstr(inreply, ep->data->datestr))) {
 #if DEBUG_THREAD
 	    fprintf(stderr, "match on date [%d]\n", ep->data->msgnum);
 #endif
@@ -400,8 +398,7 @@ struct emailinfo *hashreplylookup(int msgnum, char *inreply,
 
     ep = etable[hash(inreply)];
     while (ep != NULL) {
-	if ((msgnum != ep->data->msgnum) &&
-	    (strcmp(inreply, ep->data->subject) == 0)) {
+		if ((msgnum != ep->data->msgnum) && (strcmp(inreply, ep->data->subject) == 0)) {
 	    *issubjmatch = 1;
 #if DEBUG_THREAD
 	    fprintf(stderr, "match on subject [%d]\n", ep->data->msgnum);
@@ -447,14 +444,12 @@ struct emailinfo *hashmsgidlookup(char *msgid, int *issubjmatch)
  * found by comparing subjects, maybereply is set to 1.
 */
 
-struct emailinfo *hashreplylookup(int msgnum, char *inreply, char *subject,
-				  int *maybereply)
+struct emailinfo *hashreplylookup(int msgnum, char *inreply, char *subject, int *maybereply)
 {
     struct hashemail *ep;
 
 #if DEBUG_THREAD > 1
-    fprintf(stderr, "hashreplynumlookup(%d, '%s'...)\n", msgnum,
-	    (inreply == NULL) ? "" : inreply);
+	fprintf(stderr, "hashreplynumlookup(%d, '%s'...)\n", msgnum, (inreply == NULL) ? "" : inreply);
 #endif
     *maybereply = 0;
 
@@ -464,8 +459,7 @@ struct emailinfo *hashreplylookup(int msgnum, char *inreply, char *subject,
 	while (ep) {
 	    if (strcmp(inreply, ep->data->msgid) == 0) {
 #if DEBUG_THREAD
-		fprintf(stderr, "match on msgid   %4d %4d\n", msgnum,
-			ep->data->msgnum);
+				fprintf(stderr, "match on msgid   %4d %4d\n", msgnum, ep->data->msgnum);
 #endif
 		return ep->data;
 	    }
@@ -474,11 +468,9 @@ struct emailinfo *hashreplylookup(int msgnum, char *inreply, char *subject,
 
 	ep = etable[hash(inreply)];
 	while (ep) {
-	    if ((msgnum != ep->data->msgnum) &&
-		(strstr(inreply, ep->data->datestr))) {
+			if ((msgnum != ep->data->msgnum) && (strstr(inreply, ep->data->datestr))) {
 #if DEBUG_THREAD
-		fprintf(stderr, "match on date    %4d %4d\n", msgnum,
-			ep->data->msgnum);
+				fprintf(stderr, "match on date    %4d %4d\n", msgnum, ep->data->msgnum);
 #endif
 		return ep->data;
 	    }
@@ -487,12 +479,10 @@ struct emailinfo *hashreplylookup(int msgnum, char *inreply, char *subject,
 
 	ep = etable[hash(inreply)];
 	while (ep != NULL) {
-	    if ((msgnum != ep->data->msgnum) &&
-		(strcmp(inreply, ep->data->subject) == 0)) {
+			if ((msgnum != ep->data->msgnum) && (strcmp(inreply, ep->data->subject) == 0)) {
 		*maybereply = 1;
 #if DEBUG_THREAD
-		fprintf(stderr, "match on subject %4d %4d\n", msgnum,
-			ep->data->msgnum);
+				fprintf(stderr, "match on subject %4d %4d\n", msgnum, ep->data->msgnum);
 #endif
 		return ep->data;
 	    }
@@ -506,12 +496,25 @@ struct emailinfo *hashreplylookup(int msgnum, char *inreply, char *subject,
      * time round the loop.
      */
     {
+#define MAX_SUBJ_LEN 300
+
 	char *s, *saved_s;
 	struct emailinfo *lowest_so_far = NULL;
 	int match = 0;
 
-	s = emalloc(strlen(subject) + 1);
+        size_t subj_len;
+
+        subj_len = strlen(subject) > MAX_SUBJ_LEN ? MAX_SUBJ_LEN : strlen(subject);
+        s = emalloc(subj_len + 1);
+        saved_s = strncpy(s, subject, subj_len);
+        s[subj_len] = '\0';
+
+#ifdef NOTDEF
+/* AUDIT biege: can we trigger a DoS here by using a very long "Subject"? */
+
+	s = emalloc(strlen(subject) + 1);	
 	saved_s = strcpy(s, subject);
+#endif
 
         if (isre(s, NULL))
             do {
@@ -520,11 +523,9 @@ struct emailinfo *hashreplylookup(int msgnum, char *inreply, char *subject,
 #endif
                 ep = etable[hash(s)];
                 while (ep != NULL) {
-                    if ((strcasecmp(s, ep->data->subject) == 0) &&
-                        (msgnum != ep->data->msgnum)) {
+					if ((strcasecmp(s, ep->data->subject) == 0) && (msgnum != ep->data->msgnum)) {
                         match = 1;
-                        if (lowest_so_far == NULL
-			    || ep->data->msgnum < lowest_so_far->msgnum)
+						if (lowest_so_far == NULL || ep->data->msgnum < lowest_so_far->msgnum)
                             lowest_so_far = ep->data;
                     }
                     ep = ep->next;
@@ -538,16 +539,13 @@ struct emailinfo *hashreplylookup(int msgnum, char *inreply, char *subject,
 	    *maybereply = 1;
 	    if (lowest_so_far != NULL && lowest_so_far->msgnum < msgnum) {
 #if DEBUG_THREAD
-		fprintf(stderr, "match on extra   %4d %4d\n", msgnum,
-			lowest_so_far->msgnum);
+				fprintf(stderr, "match on extra   %4d %4d\n", msgnum, lowest_so_far->msgnum);
 #endif
 		return lowest_so_far;
 	    }
 	    else {
 #if DEBUG_THREAD
-		fprintf(stderr,
-			"match on extra   %4d %4d discarded - less than %d\n",
-			msgnum, lowest_so_far ? lowest_so_far->msgnum : -1, msgnum);
+				fprintf(stderr, "match on extra   %4d %4d discarded - less than %d\n", msgnum, lowest_so_far ? lowest_so_far->msgnum : -1, msgnum);
 #endif
 		return NULL;
 	    }
@@ -564,8 +562,7 @@ struct emailinfo *hashreplylookup(int msgnum, char *inreply, char *subject,
 ** Same as the above function, but only returns the article number.
 */
 
-int hashreplynumlookup(int msgnum, char *inreply, char *subject,
-		       int *maybereply)
+int hashreplynumlookup(int msgnum, char *inreply, char *subject, int *maybereply)
 {
     struct emailinfo *email = hashreplylookup(msgnum, inreply, subject,
 					      maybereply);
@@ -581,15 +578,19 @@ int hashreplynumlookup(int msgnum, char *inreply, char *subject,
 struct body *hashnumlookup(int num, struct emailinfo **emailp)
 {
     struct hashemail *ep;
+    struct body *lp_tmp;
     char numstr[NUMSTRLEN];
 
     sprintf(numstr, "%d", num);
-    for (ep = etable[hash(numstr)]; ep != NULL; ep = ep->next)
+    for (ep = etable[hash(numstr)]; ep != NULL; ep = ep->next) {
 	if (ep->data && (num == ep->data->msgnum)) {
 	    /* return a mere pointer to it! */
 	    *emailp = ep->data;
+	    if (!ep->data->bodylist)
+	        ep->data->bodylist = addbody(NULL, &lp_tmp, "\n", 0);
 	    return ep->data->bodylist;
 	}
+    }
     return NULL;
 }
 
@@ -663,8 +664,7 @@ struct boundary *bound(struct boundary *bnd, char *line)
 ** Add a line to a linked list that makes up an article's body.
 */
 
-struct body *addbody(struct body *bp, 
-                     struct body **lp,	/* points to the last pointer */
+struct body *addbody(struct body *bp, struct body **lp,	/* points to the last pointer */
 		     char *line, int flags)
 {
     struct body *tempnode;
@@ -728,11 +728,10 @@ int rmlastlines(struct body *bp)
 {
     struct body *tempnode;
 
-    if(NULL == bp)
+	if (NULL == bp)
       return 0;
 
-    for (tempnode = bp; tempnode->next != NULL &&
-	 (tempnode->next->line)[0] != '\0'; tempnode = tempnode->next);
+	for (tempnode = bp; tempnode->next != NULL && (tempnode->next->line)[0] != '\0'; tempnode = tempnode->next);
     if ((tempnode->line)[0] == '\n') {
 	(tempnode->line)[0] = '\0';
 	return 1;
@@ -742,13 +741,11 @@ int rmlastlines(struct body *bp)
 
 /* copy bp on to the end of origbp, deallocate bp */
 
-struct body *
-append_body(struct body *origbp, struct body **origlp, struct body *bp)
+struct body *append_body(struct body *origbp, struct body **origlp, struct body *bp)
 {
     while (bp) {
 	struct body *next;
-	origbp = addbody(origbp, origlp, bp->line,
-			 (bp->header ? BODY_HEADER : 0)
+		origbp = addbody(origbp, origlp, bp->line, (bp->header ? BODY_HEADER : 0)
 			 | (bp->html ? BODY_HTMLIZED : 0)
 			 | (bp->attached ? BODY_ATTACHED : 0));
 	next = bp->next;
@@ -775,10 +772,7 @@ void free_body(struct body *bp)
 ** the message it may be referring to is put in this list.  
 */
 
-struct reply *addreply(struct reply *rp,
-		       int fromnum,
-		       struct emailinfo *email, int maybereply,
-		       struct reply **last_node)
+struct reply *addreply(struct reply *rp, int fromnum, struct emailinfo *email, int maybereply, struct reply **last_node)
 {
     struct reply *tempnode, *newnode;
 
@@ -805,14 +799,12 @@ struct reply *addreply(struct reply *rp,
     }
     else {
 #ifdef FASTREPLYCODE
-	for (tempnode = (last_node ? *last_node : rp); tempnode->next != NULL;
-	     tempnode = tempnode->next);
+		for (tempnode = (last_node ? *last_node : rp); tempnode->next != NULL; tempnode = tempnode->next);
 	tempnode->next = newnode;
 	if (rp == threadlist && threadlist_by_msgnum && email)
 	    threadlist_by_msgnum[email->msgnum] = tempnode;
 #else
-	for (tempnode = rp; tempnode->next != NULL; tempnode =
-	     tempnode->next);
+		for (tempnode = rp; tempnode->next != NULL; tempnode = tempnode->next);
 	tempnode->next = newnode;
 #endif
     }
@@ -822,9 +814,8 @@ struct reply *addreply(struct reply *rp,
     return rp;
 }
 
-struct reply *addreply2(struct reply *rp, struct emailinfo *from_email,
-			struct emailinfo *email, int maybereply,
-			struct reply **last_node) {
+struct reply *addreply2(struct reply *rp, struct emailinfo *from_email, struct emailinfo *email, int maybereply, struct reply **last_node)
+{
 #ifdef FASTREPLYCODE
     struct reply *tempnode;
     for (tempnode = rp; tempnode != NULL; tempnode = tempnode->next) {
@@ -834,8 +825,7 @@ struct reply *addreply2(struct reply *rp, struct emailinfo *from_email,
 	    return rp;		/* don't add 2nd time */
 	}
     }
-    from_email->replylist = addreply(from_email->replylist, from_email->msgnum,
-				     email, maybereply, NULL);
+	from_email->replylist = addreply(from_email->replylist, from_email->msgnum, email, maybereply, NULL);
 #endif
     return addreply(rp, from_email->msgnum, email, maybereply, last_node);
 }
@@ -855,8 +845,7 @@ struct printed *markasprinted(struct printed *pp, int num)
     if (pp == NULL)
 	pp = newnode;
     else {
-	for (tempnode = pp; tempnode->next != NULL;
-	     tempnode = tempnode->next);
+		for (tempnode = pp; tempnode->next != NULL; tempnode = tempnode->next);
 	tempnode->next = newnode;
     }
     return pp;
@@ -881,8 +870,7 @@ int wasprinted(struct printed *list, int num)
 ** subject, or author. This is necessary for printing the index files.
 */
 
-struct header *addheader(struct header *hp,
-			 struct emailinfo *email, int sorttype, int depth)
+struct header *addheader(struct header *hp, struct emailinfo *email, int sorttype, int depth)
 {
     int isbigger;
     long yearsecs;
@@ -915,8 +903,7 @@ struct header *addheader(struct header *hp,
 	isbigger = (strcasecmp(email->name, hp->data->name) > 0) ? 0 : 1;
 	break;
     case 0:
-	isbigger = (strcasecmp(email->unre_subject,
-			       hp->data->unre_subject) > 0) ? 0 : 1;
+		isbigger = (strcasecmp(email->unre_subject, hp->data->unre_subject) > 0) ? 0 : 1;
 	break;
     case 2:
 	yearsecs = email->fromdate;
@@ -936,13 +923,12 @@ struct header *addheader(struct header *hp,
     else
 	hp->right = addheader(hp->right, email, sorttype, depth + 1);
 
-    if(sorttype == 2 && depth < max_depth/2 && !(++count_d % 3)) {
+	if (sorttype == 2 && depth < max_depth / 2 && !(++count_d % 3)) {
 	/* semi-random rebalancing */
 	struct header **hpp = (set_reverse ? &hp->left : &hp->right);
 	struct header *hp1 = *hpp;
-	if(hp1 != NULL && (hp1->right != NULL || hp1->left != NULL)) {
-	    if(hp1->right != NULL
-	       && (hp1->left == NULL || (count_d & 1))) {
+		if (hp1 != NULL && (hp1->right != NULL || hp1->left != NULL)) {
+			if (hp1->right != NULL && (hp1->left == NULL || (count_d & 1))) {
 		struct header *p = hp1->right;
 		hp1->right = p->left;
 		p->left = hp1;
@@ -960,8 +946,7 @@ struct header *addheader(struct header *hp,
     return hp;
 }
 
-struct emailsubdir *new_subdir(char *subdir, struct emailsubdir *last_subdir,
-			       char *description, time_t date)
+struct emailsubdir *new_subdir(char *subdir, struct emailsubdir *last_subdir, char *description, time_t date)
 {
     struct emailsubdir *sd = folders;
     struct emailsubdir *new_sd;
@@ -972,8 +957,7 @@ struct emailsubdir *new_subdir(char *subdir, struct emailsubdir *last_subdir,
     while (sd) {
 	if (!strcmp(subdir, sd->subdir))
 	    return sd;
-	if (sd->a_date < date
-	    && (!insert_point || sd->a_date > insert_point->a_date))
+		if (sd->a_date < date && (!insert_point || sd->a_date > insert_point->a_date))
 	    insert_point = sd;
 	if (sd->next_subdir == NULL)
 	    break;
@@ -988,12 +972,15 @@ struct emailsubdir *new_subdir(char *subdir, struct emailsubdir *last_subdir,
 	folders = new_sd;
     else if (date < folders->a_date) {
 	new_sd->next_subdir = folders;
+	folders->prior_subdir = new_sd;
 	folders = new_sd;
     }
     else {
 	new_sd->next_subdir = insert_point->next_subdir;
 	insert_point->next_subdir = new_sd;
 	new_sd->prior_subdir = insert_point;
+	if (new_sd->next_subdir)
+	    new_sd->next_subdir->prior_subdir = new_sd;
     }
     new_sd->first_email = NULL;
     new_sd->last_email = NULL;
@@ -1002,7 +989,7 @@ struct emailsubdir *new_subdir(char *subdir, struct emailsubdir *last_subdir,
     new_sd->description = description;
     new_sd->a_date = date;
     if (set_base_url != NULL) {
-	if (set_base_url[strlen(set_base_url)-1] != '/')
+		if (set_base_url[strlen(set_base_url) - 1] != '/')
 	    trio_asprintf(&new_sd->rel_path_to_top, "%s/", set_base_url);
 	else
 	    new_sd->rel_path_to_top = strsav(set_base_url);
@@ -1012,18 +999,15 @@ struct emailsubdir *new_subdir(char *subdir, struct emailsubdir *last_subdir,
 	count = 0;
 	for (p = subdir; *p; ++p)
 	    count += (*p == '/');
-	while(--count > 0) {
+		while (--count > 0) {
 	    trio_asprintf(&p, "../%s", new_sd->rel_path_to_top);
 	    free(new_sd->rel_path_to_top);
 	    new_sd->rel_path_to_top = p;
 	}
-	if (set_latest_folder != NULL && set_folder_by_date != NULL
-	    && strchr(set_folder_by_date, '/')) {
+		if (set_latest_folder != NULL && set_folder_by_date != NULL && strchr(set_folder_by_date, '/')) {
 	    static int warned = 0;
 	    if (!warned) {
-	      fprintf(stderr, "Warning: the latest_folder option combined with "
-		      "a folder_by_date option that includes a '/' will produce "
-		      "some invalid links unless you use the base_url option!!.\n");
+				fprintf(stderr, "Warning: the latest_folder option combined with " "a folder_by_date option that includes a '/' will produce " "some invalid links unless you use the base_url option!!.\n");
 	      warned = 1;
 	    }
 	}
@@ -1110,7 +1094,7 @@ int inlist_pos(struct hmlist *listname, char *wildcard)
 
 static int regex_index(struct hmlist *listname, int index)
 {
-    static int start[5] = { -1, -1, -1, -1, -1};
+	static int start[5] = { -1, -1, -1, -1, -1 };
     if (start[0] == -1) {
 	struct hmlist *tlist;
 	start[1] = start[0] = 0;
@@ -1126,11 +1110,16 @@ static int regex_index(struct hmlist *listname, int index)
 	for (tlist = set_filter_require_full_body; tlist != NULL; tlist = tlist->next)
 	    ++start[4];
     }
-    if (index == -1) return start[4];
-    if (listname == set_filter_out) return start[0] + index;
-    if (listname == set_filter_require) return start[1] + index;
-    if (listname == set_filter_out_full_body) return start[2] + index;
-    if (listname == set_filter_require_full_body) return start[3] + index;
+	if (index == -1)
+		return start[4];
+	if (listname == set_filter_out)
+		return start[0] + index;
+	if (listname == set_filter_require)
+		return start[1] + index;
+	if (listname == set_filter_out_full_body)
+		return start[2] + index;
+	if (listname == set_filter_require_full_body)
+		return start[3] + index;
     return -1;
 }
 
@@ -1154,8 +1143,8 @@ int inlist_regex_pos(struct hmlist *listname, char *str)
 	if (!pcre_list) {
 	    int n = regex_index(NULL, -1);
 	    int i;
-	    pcre_list = (pcre **)emalloc(n * sizeof(pcre *));
-	    extra_list = (pcre_extra **)emalloc(n * sizeof(pcre_extra *));
+			pcre_list = (pcre **) emalloc(n * sizeof(pcre *));
+			extra_list = (pcre_extra **) emalloc(n * sizeof(pcre_extra *));
 	    for (i = 0; i < n; ++i) {
 	        pcre_list[i] = NULL;
 	        extra_list[i] = NULL;
@@ -1164,14 +1153,12 @@ int inlist_regex_pos(struct hmlist *listname, char *str)
 	if ((p = pcre_list[index]) == NULL) {
 	    p = pcre_compile(tlist->val, 0, &errptr, &epos, NULL);
 	    if (!p) {
-	        sprintf(errmsg, "Error at position %d of regular expression '%s': %s",
-			epos, tlist->val, errptr);
+		snprintf(errmsg, sizeof(errmsg), "Error at position %d of regular expression '%s': %s", epos, tlist->val, errptr);
 		progerr(errmsg);
 	    }
 	    extra = pcre_study(p, 0, &errptr);
 	    if (errptr) {
-	        sprintf(errmsg, "Error studying regular expression '%s': %s",
-			tlist->val, errptr);
+		snprintf(errmsg, sizeof(errmsg), "Error studying regular expression '%s': %s", tlist->val, errptr);
 		progerr(errmsg);
 	    }
 	    pcre_list[index] = p;
