@@ -19,6 +19,8 @@ struct body *hashnumlookup(int, struct emailinfo **);
 struct emailinfo *neighborlookup(int, int);
 
 struct body *addbody(struct body *, struct body **, char *, int);
+struct body *append_body(struct body *, struct body **, struct body *);
+void free_body(struct body *);
 
 struct reply *addreply(struct reply *, int, struct emailinfo *, int,
 		       struct reply **);
@@ -26,7 +28,7 @@ struct reply *addreply2(struct reply *, struct emailinfo *, struct emailinfo *,
 			int, struct reply **);
 int rmlastlines(struct body *);
 
-struct emailsubdir *new_subdir(char *, struct emailsubdir *, char *);
+struct emailsubdir *new_subdir(char *, struct emailsubdir *, char *, time_t);
 
 struct printed *markasprinted(struct printed *, int);
 int wasprinted(struct printed *, int);
