@@ -1,6 +1,6 @@
 /*************************************************************************
  *
- * $Id: trionan.h,v 1.1 2001/11/22 13:27:07 daniel Exp $
+ * $Id: trionan.h,v 1.2 2003/01/23 01:27:32 pcm Exp $
  *
  * Copyright (C) 2001 Bjorn Reese <breese@users.sourceforge.net>
  *
@@ -15,42 +15,67 @@
  *
  ************************************************************************/
 
-#ifndef __TRIO_NAN_H__
-#define __TRIO_NAN_H__
+#ifndef TRIO_NAN_H
+#define TRIO_NAN_H
 
 #include "triodef.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
+
+enum {
+  TRIO_FP_INFINITE,
+  TRIO_FP_NAN,
+  TRIO_FP_NORMAL,
+  TRIO_FP_SUBNORMAL,
+  TRIO_FP_ZERO
+};
+
 /*
  * Return NaN (Not-a-Number).
  */
-TRIO_PUBLIC double trio_nan(void);
+TRIO_PUBLIC double trio_nan TRIO_PROTO((void));
 
 /*
  * Return positive infinity.
  */
-TRIO_PUBLIC double trio_pinf(void);
+TRIO_PUBLIC double trio_pinf TRIO_PROTO((void));
 
 /*
  * Return negative infinity.
  */
-TRIO_PUBLIC double trio_ninf(void);
+TRIO_PUBLIC double trio_ninf TRIO_PROTO((void));
   
+/*
+ * Return negative zero.
+ */
+TRIO_PUBLIC double trio_nzero TRIO_PROTO((TRIO_NOARGS));
+
 /*
  * If number is a NaN return non-zero, otherwise return zero.
  */
-TRIO_PUBLIC int trio_isnan(double number);
+TRIO_PUBLIC int trio_isnan TRIO_PROTO((double number));
 
 /*
  * If number is positive infinity return 1, if number is negative
  * infinity return -1, otherwise return 0.
  */
-TRIO_PUBLIC int trio_isinf(double number);
+TRIO_PUBLIC int trio_isinf TRIO_PROTO((double number));
+
+/*
+ * If number is finite return non-zero, otherwise return zero.
+ */
+TRIO_PUBLIC int trio_isfinite TRIO_PROTO((double number));
+
+TRIO_PUBLIC int trio_signbit TRIO_PROTO((double number));
+
+TRIO_PUBLIC int trio_fpclassify TRIO_PROTO((double number));
+
+TRIO_PUBLIC int trio_fpclassify_and_signbit TRIO_PROTO((double number, int *is_negative));
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* __TRIO_NAN_H__ */
+#endif /* TRIO_NAN_H */
